@@ -4,7 +4,7 @@
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -20,8 +20,9 @@
 #define HEADERS_H_
 
 #include "belle-sip/defs.h"
-#include "belle-sip/uri.h"
+#include "belle-sip/sip-uri.h"
 #include "belle-sip/utils.h"
+#include "belle-sip/parameters.h"
 
 #include <time.h>
 
@@ -74,8 +75,9 @@ BELLESIP_EXPORT void belle_sip_header_address_set_displayname(belle_sip_header_a
  *
  **************************************************************************************/
 
-typedef struct _belle_sip_header belle_sip_header_t;
+BELLESIP_EXPORT belle_sip_header_t* belle_sip_header_parse (const char* header);
 BELLESIP_EXPORT belle_sip_header_t* belle_sip_header_create (const char* name,const char* value);
+BELLESIP_EXPORT belle_sip_header_t* belle_http_header_create (const char* name,const char* value);
 BELLESIP_EXPORT const char* belle_sip_header_get_name (const belle_sip_header_t* obj);
 BELLESIP_EXPORT void belle_sip_header_set_name (belle_sip_header_t* obj,const char* value);
 BELLESIP_EXPORT belle_sip_error_code belle_sip_header_marshal(belle_sip_header_t* header, char* buff, size_t buff_size, size_t *offset);
@@ -377,6 +379,7 @@ BELLESIP_EXPORT unsigned char belle_sip_header_record_route_get_auto_outgoing(co
  BELLESIP_EXPORT void belle_sip_header_user_agent_set_products(belle_sip_header_user_agent_t* user_agent,belle_sip_list_t* value);
  BELLESIP_EXPORT void belle_sip_header_user_agent_add_product(belle_sip_header_user_agent_t* user_agent,const char* product);
  #define BELLE_SIP_HEADER_USER_AGENT(t) BELLE_SIP_CAST(t,belle_sip_header_user_agent_t)
+#define BELLE_SIP_USER_AGENT "User-Agent"
 
  /******************************
  * Content length inherent from object
@@ -441,6 +444,20 @@ BELLESIP_EXPORT belle_sip_header_proxy_authorization_t* belle_sip_header_proxy_a
 #define BELLE_SIP_HEADER_PROXY_AUTHORIZATION(t) BELLE_SIP_CAST(t,belle_sip_header_proxy_authorization_t)
 #define BELLE_SIP_PROXY_AUTHORIZATION "Proxy-Authorization"
 
+/*******************************
+ * http_authorization inherit from Authorization
+ */
+typedef struct _belle_http_header_authorization belle_http_header_authorization_t;
+BELLESIP_EXPORT belle_http_header_authorization_t* belle_http_header_authorization_new();
+/*cannot be parsed for now
+BELLESIP_EXPORT belle_http_header_authorization_t* belle_http_header_authorization_parse(const char* proxy_authorization);
+*/
+BELLESIP_EXPORT void belle_http_header_authorization_set_uri(belle_http_header_authorization_t* authorization, belle_generic_uri_t* uri);
+BELLESIP_EXPORT belle_generic_uri_t* belle_http_header_authorization_get_uri(const belle_http_header_authorization_t* authorization);
+#define BELLE_HTTP_HEADER_AUTHORIZATION(t) BELLE_SIP_CAST(t,belle_http_header_authorization_t)
+#define BELLE_HTTP_AUTHORIZATION "Authorization"
+
+
 /*******************************
  * www_authenticate inherit from parameters
  */
@@ -477,20 +494,6 @@ BELLESIP_EXPORT belle_sip_header_proxy_authenticate_t* belle_sip_header_proxy_au
 #define BELLE_SIP_HEADER_PROXY_AUTHENTICATE(t) BELLE_SIP_CAST(t,belle_sip_header_proxy_authenticate_t)
 #define BELLE_SIP_PROXY_AUTHENTICATE "Proxy-Authenticate"
 
-/******************************
- *
- * Extension header inherit from header
- *
- ******************************/
-typedef struct _belle_sip_header_extension belle_sip_header_extension_t;
-
-BELLESIP_EXPORT belle_sip_header_extension_t* belle_sip_header_extension_new();
-
-BELLESIP_EXPORT belle_sip_header_extension_t* belle_sip_header_extension_parse (const char* extension) ;
-BELLESIP_EXPORT belle_sip_header_extension_t* belle_sip_header_extension_create (const char* name,const char* value);
-BELLESIP_EXPORT const char* belle_sip_header_extension_get_value(const belle_sip_header_extension_t* extension);
-BELLESIP_EXPORT void belle_sip_header_extension_set_value(belle_sip_header_extension_t* extension,const char* value);
-#define BELLE_SIP_HEADER_EXTENSION(t) BELLE_SIP_CAST(t,belle_sip_header_extension_t)
 /******************************
  *
  * Max forward inherit from header
