@@ -4,7 +4,7 @@
 
 	This program is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
-	the Free Software Foundation, either version 3 of the License, or
+	the Free Software Foundation, either version 2 of the License, or
 	(at your option) any later version.
 
 	This program is distributed in the hope that it will be useful,
@@ -419,7 +419,7 @@ belle_sip_object_pool_t * belle_sip_object_pool_get_current();
 int belle_sip_object_pool_cleanable(belle_sip_object_pool_t *pool);
 void belle_sip_object_pool_clean(belle_sip_object_pool_t *obj);
 
-BELLE_SIP_DECLARE_VPTR(belle_sip_object_t);
+BELLE_SIP_DECLARE_VPTR(belle_sip_object_t)
 
 BELLE_SIP_END_DECLS
 
