@@ -4,7 +4,7 @@
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -23,8 +23,9 @@
 #include <stdlib.h>
 #include <string.h>
 #include <stdarg.h>
-#include "belle_sip_messageParser.h"
-#include "belle_sip_messageLexer.h"
+#include "grammars/belle_sip_messageLexer.h"
+#include "grammars/belle_sip_messageParser.h"
+
 #include "belle_sip_internal.h"
 #include "listeningpoint_internal.h"
 
@@ -33,9 +34,78 @@
  ***********************/
 
 GET_SET_STRING(belle_sip_header,name);
+#define PROTO_SIP 0x1
+#define PROTO_HTTP 0x1<<1
+typedef belle_sip_header_t* (*header_parse_func)(const char*) ;
+struct header_name_func_pair {
+	int protocol;
+	const char* name;
+	header_parse_func func;
+};
+static struct header_name_func_pair  header_table[] = {
+	 {PROTO_SIP, 			"m",							(header_parse_func)belle_sip_header_contact_parse}
+	,{PROTO_SIP, 			BELLE_SIP_CONTACT,				(header_parse_func)belle_sip_header_contact_parse}
+	,{PROTO_SIP, 			"f",							(header_parse_func)belle_sip_header_from_parse}
+	,{PROTO_SIP, 			BELLE_SIP_FROM,					(header_parse_func)belle_sip_header_from_parse}
+	,{PROTO_SIP, 			"t",							(header_parse_func)belle_sip_header_to_parse}
+	,{PROTO_SIP, 			BELLE_SIP_TO,					(header_parse_func)belle_sip_header_to_parse}
+	,{PROTO_SIP, 			"i",							(header_parse_func)belle_sip_header_call_id_parse}
+	,{PROTO_SIP, 			BELLE_SIP_CALL_ID,				(header_parse_func)belle_sip_header_call_id_parse}
+	,{PROTO_SIP, 			"l",							(header_parse_func)belle_sip_header_content_length_parse}
+	,{PROTO_SIP|PROTO_HTTP, BELLE_SIP_CONTENT_LENGTH,		(header_parse_func)belle_sip_header_content_length_parse}
+	,{PROTO_SIP, 			"c",							(header_parse_func)belle_sip_header_content_type_parse}
+	,{PROTO_SIP|PROTO_HTTP, BELLE_SIP_CONTENT_TYPE,			(header_parse_func)belle_sip_header_content_type_parse}
+	,{PROTO_SIP, 			BELLE_SIP_CSEQ,					(header_parse_func)belle_sip_header_cseq_parse}
+	,{PROTO_SIP, 			BELLE_SIP_ROUTE,				(header_parse_func)belle_sip_header_route_parse}
+	,{PROTO_SIP, 			BELLE_SIP_RECORD_ROUTE,			(header_parse_func)belle_sip_header_record_route_parse}
+	,{PROTO_SIP, 			"v",							(header_parse_func)belle_sip_header_via_parse}
+	,{PROTO_SIP, 			BELLE_SIP_VIA,					(header_parse_func)belle_sip_header_via_parse}
+	,{PROTO_SIP, 			BELLE_SIP_AUTHORIZATION,		(header_parse_func)belle_sip_header_authorization_parse}
+	,{PROTO_SIP, 			BELLE_SIP_PROXY_AUTHORIZATION,	(header_parse_func)belle_sip_header_proxy_authorization_parse}
+	,{PROTO_SIP|PROTO_HTTP,	BELLE_SIP_WWW_AUTHENTICATE,		(header_parse_func)belle_sip_header_www_authenticate_parse}
+	,{PROTO_SIP|PROTO_HTTP,	BELLE_SIP_PROXY_AUTHENTICATE,	(header_parse_func)belle_sip_header_proxy_authenticate_parse}
+	,{PROTO_SIP, 			BELLE_SIP_MAX_FORWARDS,			(header_parse_func)belle_sip_header_max_forwards_parse}
+	,{PROTO_SIP|PROTO_HTTP, BELLE_SIP_USER_AGENT,			(header_parse_func)belle_sip_header_user_agent_parse}
+	,{PROTO_SIP, 			BELLE_SIP_EXPIRES,				(header_parse_func)belle_sip_header_expires_parse}
+	,{PROTO_SIP|PROTO_HTTP, BELLE_SIP_ALLOW,				(header_parse_func)belle_sip_header_allow_parse}
+	,{PROTO_SIP, 			BELLE_SIP_SUBSCRIPTION_STATE,	(header_parse_func)belle_sip_header_subscription_state_parse}
+	,{PROTO_SIP, 			BELLE_SIP_SERVICE_ROUTE,		(header_parse_func)belle_sip_header_service_route_parse}
+	,{PROTO_SIP, 			BELLE_SIP_REFER_TO,				(header_parse_func)belle_sip_header_refer_to_parse}
+	,{PROTO_SIP, 			BELLE_SIP_REFERRED_BY,			(header_parse_func)belle_sip_header_referred_by_parse}
+	,{PROTO_SIP, 			BELLE_SIP_REPLACES,				(header_parse_func)belle_sip_header_replaces_parse}
+	,{PROTO_SIP, 			BELLE_SIP_DATE,					(header_parse_func)belle_sip_header_date_parse}
+	,{PROTO_SIP, 			BELLE_SIP_P_PREFERRED_IDENTITY,	(header_parse_func)belle_sip_header_p_preferred_identity_parse}
+	,{PROTO_SIP, 			BELLE_SIP_PRIVACY,				(header_parse_func)belle_sip_header_privacy_parse}
+};
 
-belle_sip_header_t* belle_sip_header_create (const char* name,const char* value) {
+static belle_sip_header_t* belle_header_create(const char* name,const char* value,int protocol) {
+	int i;
+	belle_sip_header_t* ret;
+	size_t elements =sizeof(header_table)/sizeof(struct header_name_func_pair);
+
+	if (!name || name[0]=='0') {
+		belle_sip_error("Cannot create header without name");
+		return NULL;
+	}
+
+	for(i=0;i<elements;i++) {
+		if ((header_table[i].protocol & protocol) && strcasecmp(header_table[i].name,name)==0) {
+			char* raw = belle_sip_strdup_printf("%s:%s",name,value);
+			ret=header_table[i].func(raw);
+			belle_sip_free(raw);
+			return ret;
+		}
+	}
+	/*not a known header*/
 	return BELLE_SIP_HEADER(belle_sip_header_extension_create(name,value));
+
+}
+belle_sip_header_t* belle_sip_header_create(const char* name, const char* value) {
+	return belle_header_create(name,value,PROTO_SIP);
+}
+
+belle_sip_header_t* belle_http_header_create (const char* name,const char* value) {
+	return belle_header_create(name,value,PROTO_HTTP);
 }
 
 void belle_sip_header_init(belle_sip_header_t *header) {
@@ -66,20 +136,22 @@ belle_sip_header_t* belle_sip_header_get_next(const belle_sip_header_t* header)
 }
 
 const char *belle_sip_header_get_unparsed_value(belle_sip_header_t* obj){
-	char *tmp=belle_sip_object_to_string(obj);
-	char *ret;
-	char *end;
-	if (obj->unparsed_value){
-		belle_sip_free(obj->unparsed_value);
-		obj->unparsed_value=NULL;
+	if (BELLE_SIP_OBJECT_IS_INSTANCE_OF(obj,belle_sip_header_extension_t)) {
+		return belle_sip_header_extension_get_value(BELLE_SIP_HEADER_EXTENSION(obj));
+	} else {
+		char *tmp=belle_sip_object_to_string(obj);
+		char *ret;
+		char *end;
+		if (obj->unparsed_value){
+			belle_sip_free(obj->unparsed_value);
+			obj->unparsed_value=NULL;
+		}
+		obj->unparsed_value=tmp;
+		ret=tmp;
+		ret+=strlen(obj->name)+1; /* name + semicolon*/
+		for(;*ret==' ';ret++){};/*skip spaces*/
+		return ret;
 	}
-	obj->unparsed_value=tmp;
-	ret=tmp;
-	ret+=strlen(obj->name)+1; /* name + semicolon*/
-	for(;*ret==' ';ret++){};/*skip spaces*/
-	end=strchr(ret,'\r');
-	if (end) *end='\0'; /*remove \r\n*/
-	return ret;
 }
 
 belle_sip_error_code belle_sip_header_marshal(belle_sip_header_t* header, char* buff, size_t buff_size, size_t *offset) {
@@ -95,6 +167,8 @@ BELLE_SIP_DECLARE_NO_IMPLEMENTED_INTERFACES(belle_sip_header_t);
 
 BELLE_SIP_INSTANCIATE_VPTR(belle_sip_header_t,belle_sip_object_t,belle_sip_header_destroy,belle_sip_header_clone,belle_sip_header_marshal,TRUE);
 
+BELLE_SIP_PARSE(header)
+
 
 /************************
  * header_address
@@ -121,7 +195,7 @@ static void belle_sip_header_address_clone(belle_sip_header_address_t *addr, con
 	}
 }
 
-belle_sip_error_code belle_sip_header_address_marshal(belle_sip_header_address_t* header, char* buff, size_t buff_size, size_t *offset) {
+static belle_sip_error_code _belle_sip_header_address_marshal(belle_sip_header_address_t* header, char* buff, size_t buff_size, size_t *offset, int force_angle_quote) {
 	belle_sip_error_code error=BELLE_SIP_OK;
 	/*1 display name*/
 	if (header->displayname) {
@@ -130,7 +204,7 @@ belle_sip_error_code belle_sip_header_address_marshal(belle_sip_header_address_t
 	}
 	if (header->uri) {
 		/*cases where < is required*/
-		if (header->displayname
+		if (force_angle_quote || header->displayname
 			|| belle_sip_parameters_get_parameter_names((belle_sip_parameters_t*)header->uri)
 			|| belle_sip_uri_get_header_names(header->uri)
 			|| belle_sip_parameters_get_parameter_names(&header->base)) {
@@ -139,7 +213,7 @@ belle_sip_error_code belle_sip_header_address_marshal(belle_sip_header_address_t
 		}
 		error=belle_sip_uri_marshal(header->uri,buff,buff_size,offset);
 		if (error!=BELLE_SIP_OK) return error;
-		if (header->displayname
+		if (force_angle_quote || header->displayname
 				|| belle_sip_parameters_get_parameter_names((belle_sip_parameters_t*)header->uri)
 				|| belle_sip_uri_get_header_names(header->uri)
 				|| belle_sip_parameters_get_parameter_names(&header->base)) {
@@ -152,6 +226,10 @@ belle_sip_error_code belle_sip_header_address_marshal(belle_sip_header_address_t
 	return error;
 }
 
+belle_sip_error_code belle_sip_header_address_marshal(belle_sip_header_address_t* header, char* buff, size_t buff_size, size_t *offset){
+	return _belle_sip_header_address_marshal(header, buff, buff_size, offset, FALSE);
+}
+
 BELLE_SIP_NEW_HEADER(header_address,parameters,"header_address")
 BELLE_SIP_PARSE(header_address)
 GET_SET_STRING(belle_sip_header_address,displayname);
@@ -308,10 +386,10 @@ int belle_sip_header_contact_is_unknown(const belle_sip_header_contact_t *a){
 * From header object inherent from header_address
 ****************************
 */
-#define BELLE_SIP_FROM_LIKE_MARSHAL(header) \
-		belle_sip_error_code error=belle_sip_##header_marshal(BELLE_SIP_HEADER(header), buff, buff_size, offset);\
+#define BELLE_SIP_FROM_LIKE_MARSHAL(header,force_angle_quote) \
+		belle_sip_error_code error=belle_sip_header_marshal(BELLE_SIP_HEADER(header), buff, buff_size, offset);\
 		if (error!=BELLE_SIP_OK) return error;\
-		error=belle_sip_header_address_marshal(&header->address, buff, buff_size, offset); \
+		error=_belle_sip_header_address_marshal(&header->address, buff, buff_size, offset, force_angle_quote); \
 		if (error!=BELLE_SIP_OK) return error;\
 		return error;
 
@@ -326,7 +404,7 @@ static void belle_sip_header_from_clone(belle_sip_header_from_t* from, const bel
 }
 
 belle_sip_error_code belle_sip_header_from_marshal(belle_sip_header_from_t* from, char* buff, size_t buff_size, size_t *offset) {
-	BELLE_SIP_FROM_LIKE_MARSHAL(from);
+	BELLE_SIP_FROM_LIKE_MARSHAL(from,FALSE);
 }
 
 belle_sip_header_from_t* belle_sip_header_from_create2(const char *uri, const char *tag){
@@ -391,7 +469,7 @@ void belle_sip_header_to_clone(belle_sip_header_to_t *contact, const belle_sip_h
 }
 
 belle_sip_error_code belle_sip_header_to_marshal(belle_sip_header_to_t* to, char* buff, size_t buff_size, size_t *offset) {
-	BELLE_SIP_FROM_LIKE_MARSHAL(to)
+	BELLE_SIP_FROM_LIKE_MARSHAL(to,FALSE)
 }
 
 BELLE_SIP_NEW_HEADER(header_to,header_address,BELLE_SIP_TO)
@@ -761,7 +839,7 @@ static void belle_sip_header_route_clone(belle_sip_header_route_t* route, const
 }
 
 belle_sip_error_code belle_sip_header_route_marshal(belle_sip_header_route_t* route, char* buff, size_t buff_size, size_t *offset) {
-	BELLE_SIP_FROM_LIKE_MARSHAL(route)
+	BELLE_SIP_FROM_LIKE_MARSHAL(route,TRUE)
 }
 
 BELLE_SIP_NEW_HEADER(header_route,header_address,BELLE_SIP_ROUTE)
@@ -790,7 +868,7 @@ static void belle_sip_header_record_route_clone(belle_sip_header_record_route_t*
 }
 
 belle_sip_error_code belle_sip_header_record_route_marshal(belle_sip_header_record_route_t* record_route, char* buff, size_t buff_size, size_t *offset) {
-	BELLE_SIP_FROM_LIKE_MARSHAL(record_route)
+	BELLE_SIP_FROM_LIKE_MARSHAL(record_route,TRUE)
 }
 
 belle_sip_header_record_route_t *belle_sip_header_record_route_new_auto_outgoing() {
@@ -821,7 +899,7 @@ static void belle_sip_header_service_route_clone(belle_sip_header_service_route_
 }
 
 belle_sip_error_code belle_sip_header_service_route_marshal(belle_sip_header_service_route_t* service_route, char* buff, size_t buff_size, size_t *offset) {
-	BELLE_SIP_FROM_LIKE_MARSHAL(service_route)
+	BELLE_SIP_FROM_LIKE_MARSHAL(service_route,TRUE)
 }
 
 BELLE_SIP_NEW_HEADER(header_service_route,header_address,BELLE_SIP_SERVICE_ROUTE)
@@ -927,27 +1005,7 @@ belle_sip_header_extension_t* belle_sip_header_extension_create (const char* nam
 	return ext;
 
 }
-/**
- * special case for this header. I don't know why
- */
-belle_sip_header_extension_t* belle_sip_header_extension_parse (const char* value) {
-	pANTLR3_INPUT_STREAM           input;
-	pbelle_sip_messageLexer               lex;
-	pANTLR3_COMMON_TOKEN_STREAM    tokens;
-	pbelle_sip_messageParser              parser;
-	belle_sip_messageParser_header_extension_return l_parsed_object;
-	input  = ANTLR_STREAM_NEW("header_extension",value,strlen(value));
-	lex    = belle_sip_messageLexerNew                (input);
-	tokens = antlr3CommonTokenStreamSourceNew  (1025, lex->pLexer->rec->state->tokSource);
-	parser = belle_sip_messageParserNew               (tokens);
-	l_parsed_object = parser->header_extension(parser,FALSE);
-	parser ->free(parser);
-	tokens ->free(tokens);
-	lex    ->free(lex);
-	input  ->close(input);
-	if (l_parsed_object.ret == NULL) belle_sip_error("Parser error for [%s]",value);\
-	return BELLE_SIP_HEADER_EXTENSION(l_parsed_object.ret);
-}
+
 GET_SET_STRING(belle_sip_header_extension,value);
 /**************************
 *Authorization header object inherent from parameters
@@ -1003,17 +1061,23 @@ GET_SET_STRING(belle_sip_header_extension,value);
 		error=belle_sip_snprintf(buff,buff_size,offset,"%srealm=\"%s\"",border,header->realm);\
 		if (error!=BELLE_SIP_OK) return error;\
 		border=", ";\
-		}\
+	}\
 	if (header->nonce) {\
 		error=belle_sip_snprintf(buff,buff_size,offset,"%snonce=\"%s\"",border,header->nonce);\
 		if (error!=BELLE_SIP_OK) return error;\
 		border=", ";\
-		}\
+	}\
 	if (header->algorithm) {\
-		error=belle_sip_snprintf(buff,buff_size,offset,"%salgorithm=%s",border,header->algorithm);\
+		const char* format;\
+		if (BELLE_SIP_OBJECT_IS_INSTANCE_OF(header,belle_http_header_authorization_t)) {\
+			format="%salgorithm=\"%s\"";\
+		} else {\
+			format="%salgorithm=%s";\
+		}\
+		error=belle_sip_snprintf(buff,buff_size,offset,format,border,header->algorithm);\
 		if (error!=BELLE_SIP_OK) return error;\
 		border=", ";\
-		}\
+	}\
 	if (header->opaque) {\
 		error=belle_sip_snprintf(buff,buff_size,offset,"%sopaque=\"%s\"",border,header->opaque);\
 		if (error!=BELLE_SIP_OK) return error;\
@@ -1086,11 +1150,7 @@ belle_sip_error_code belle_sip_header_authorization_marshal(belle_sip_header_aut
 		error=belle_sip_snprintf(buff,buff_size,offset,"%s","\"");
 		if (error!=BELLE_SIP_OK) return error;
 	}
-	if (authorization->algorithm) {
-		error=belle_sip_snprintf(buff,buff_size,offset,"%salgorithm=%s",border,authorization->algorithm);
-		if (error!=BELLE_SIP_OK) return error;
-		border=", ";
-	}
+
 	if (authorization->response) {
 		error=belle_sip_snprintf(buff,buff_size,offset,"%sresponse=\"%s\"",border,authorization->response);
 		if (error!=BELLE_SIP_OK) return error;
@@ -1108,7 +1168,13 @@ belle_sip_error_code belle_sip_header_authorization_marshal(belle_sip_header_aut
 		border=", ";
 	}
 	if (authorization->qop) {
-		error=belle_sip_snprintf(buff,buff_size,offset,"%sqop=%s",border,authorization->qop);
+		const char* format;
+		if (BELLE_SIP_OBJECT_IS_INSTANCE_OF(authorization,belle_http_header_authorization_t)) {
+			format="%sqop=\"%s\"";
+		} else {
+			format="%sqop=%s";
+		}
+		error=belle_sip_snprintf(buff,buff_size,offset,format,border,authorization->qop);
 		if (error!=BELLE_SIP_OK) return error;
 	}
 	return error;
@@ -1147,6 +1213,7 @@ struct _belle_sip_header_proxy_authorization  {
 
 
 static void belle_sip_header_proxy_authorization_destroy(belle_sip_header_proxy_authorization_t* proxy_authorization) {
+
 }
 
 static void belle_sip_header_proxy_authorization_clone(belle_sip_header_proxy_authorization_t* proxy_authorization,
@@ -1159,6 +1226,61 @@ belle_sip_error_code belle_sip_header_proxy_authorization_marshal(belle_sip_head
 
 BELLE_SIP_NEW_HEADER(header_proxy_authorization,header_authorization,BELLE_SIP_PROXY_AUTHORIZATION)
 BELLE_SIP_PARSE(header_proxy_authorization)
+/**************************
+*HTTP Authorization header object inherent from Authorization
+****************************
+*/
+struct _belle_http_header_authorization  {
+	belle_sip_header_authorization_t authorization;
+	belle_generic_uri_t* uri;
+};
+
+static void belle_http_header_authorization_init(belle_http_header_authorization_t* authorization) {
+	belle_sip_header_set_name(BELLE_SIP_HEADER(authorization),BELLE_HTTP_AUTHORIZATION);
+}
+static void belle_http_header_authorization_destroy(belle_http_header_authorization_t* authorization) {
+	if (authorization->uri) {
+		belle_sip_object_unref(authorization->uri);
+	}
+}
+
+static void belle_http_header_authorization_clone(belle_http_header_authorization_t* authorization,
+                                                 const belle_http_header_authorization_t *orig ) {
+	if (belle_http_header_authorization_get_uri(orig)) {
+		belle_http_header_authorization_set_uri(authorization,BELLE_GENERIC_URI(belle_sip_object_clone(BELLE_SIP_OBJECT(belle_http_header_authorization_get_uri(orig)))));
+	}
+}
+
+belle_sip_error_code belle_http_header_authorization_marshal(belle_http_header_authorization_t* authorization, char* buff, size_t buff_size, size_t *offset) {
+	belle_sip_error_code error=BELLE_SIP_OK;
+
+	/*first make sure there is no sip uri*/
+	if (belle_sip_header_authorization_get_uri(BELLE_SIP_HEADER_AUTHORIZATION(authorization))) {
+		belle_sip_error ("Cannot marshal http_header_authorization because a sip uri is set. Use belle_http_authorization_set uri instead of belle_sip_header_authorization_set_uri");
+		return BELLE_SIP_NOT_IMPLEMENTED;
+	}
+	belle_sip_header_authorization_marshal(BELLE_SIP_HEADER_AUTHORIZATION(authorization),buff,buff_size,offset);
+	if (authorization->uri) {
+		error=belle_sip_snprintf(buff,buff_size,offset,", uri=\"");
+		if (error!=BELLE_SIP_OK) return error;
+		error=belle_generic_uri_marshal(authorization->uri,buff,buff_size,offset);
+		if (error!=BELLE_SIP_OK) return error;
+		error=belle_sip_snprintf(buff,buff_size,offset,"%s","\"");
+		if (error!=BELLE_SIP_OK) return error;
+	}
+	return error;
+}
+
+BELLE_NEW(belle_http_header_authorization,belle_sip_header_authorization)
+belle_generic_uri_t* belle_http_header_authorization_get_uri(const belle_http_header_authorization_t* authorization) {
+	return authorization->uri;
+}
+void belle_http_header_authorization_set_uri( belle_http_header_authorization_t* authorization,belle_generic_uri_t* uri) {
+	if (authorization->uri) belle_sip_object_unref(authorization->uri);
+	if (uri) belle_sip_object_ref(uri);
+	authorization->uri=uri;
+}
+
 /**************************
 *WWW-Authenticate header object inherent from parameters
 ****************************
@@ -1355,7 +1477,7 @@ static void belle_sip_header_##name##_destroy(belle_sip_header_##name##_t * obj)
 } \
 void belle_sip_header_##name##_clone(belle_sip_header_##name##_t *contact, const belle_sip_header_##name##_t *orig){ }\
 belle_sip_error_code belle_sip_header_##name##_marshal(belle_sip_header_##name##_t* name, char* buff, size_t buff_size, size_t *offset) {\
-	BELLE_SIP_FROM_LIKE_MARSHAL(name)\
+	BELLE_SIP_FROM_LIKE_MARSHAL(name,FALSE)\
 }\
 BELLE_SIP_NEW_HEADER(header_##name,header_address,header_name)\
 BELLE_SIP_PARSE(header_##name)\
