@@ -4,7 +4,7 @@
 
 	This program is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
-	the Free Software Foundation, either version 3 of the License, or
+	the Free Software Foundation, either version 2 of the License, or
 	(at your option) any later version.
 
 	This program is distributed in the hope that it will be useful,
@@ -63,7 +63,6 @@
 
 #endif
 
-#include "port.h"
 
 /*etc*/
 
@@ -107,6 +106,10 @@
 			method(__obj,arg1,arg2,arg3);\
 			__BELLE_SIP_INVOKE_LISTENER_END
 
+#define BELLE_SIP_INVOKE_LISTENER_ARG(listener,interface_name,method,arg) \
+	((BELLE_SIP_INTERFACE_GET_METHODS((listener),interface_name)->method!=NULL)  ? \
+		BELLE_SIP_INTERFACE_GET_METHODS((listener),interface_name)->method(listener,(arg)) : 0 )
+			
 typedef struct weak_ref{
 	struct weak_ref *next;
 	belle_sip_object_destroy_notify_t notify;
@@ -190,7 +193,14 @@ BELLE_SIP_DECLARE_VPTR(belle_sip_certificates_chain_t);
 BELLE_SIP_DECLARE_VPTR(belle_sip_signing_key_t);
 BELLE_SIP_DECLARE_VPTR(belle_sip_dns_srv_t);
 BELLE_SIP_DECLARE_VPTR(belle_sip_dict_t)
-
+BELLE_SIP_DECLARE_VPTR(belle_http_provider_t);
+BELLE_SIP_DECLARE_VPTR(belle_http_channel_context_t);
+BELLE_SIP_DECLARE_VPTR(belle_http_request_t);
+BELLE_SIP_DECLARE_VPTR(belle_http_response_t);
+BELLE_SIP_DECLARE_VPTR(belle_generic_uri_t);
+BELLE_SIP_DECLARE_VPTR(belle_http_callbacks_t);
+BELLE_SIP_DECLARE_VPTR(belle_tls_verify_policy_t);
+BELLE_SIP_DECLARE_VPTR(belle_http_header_authorization_t);
 
 BELLE_SIP_DECLARE_CUSTOM_VPTR_BEGIN(belle_sip_resolver_context_t,belle_sip_source_t)
 	void (*cancel)(belle_sip_resolver_context_t *);
@@ -367,7 +377,7 @@ BELLESIP_INTERNAL_EXPORT unsigned int belle_sip_random(void);
 		belle_sip_parameters_set_parameter(BELLE_SIP_PARAMETERS(obj),#attribute,NULL);\
 	}
 
-#if HAVE_ANTLR_STRING_STREAM_NEW
+#ifdef HAVE_ANTLR_STRING_STREAM_NEW
 #define ANTLR_STREAM_NEW(object_type, value,length) \
 antlr3StringStreamNew((pANTLR3_UINT8)value,ANTLR3_ENC_8BIT,(ANTLR3_UINT32)length,(pANTLR3_UINT8)#object_type)
 #else
@@ -376,17 +386,17 @@ antlr3NewAsciiStringCopyStream((pANTLR3_UINT8)value,(ANTLR3_UINT32)length,NULL)
 #endif /*HAVE_ANTLR_STRING_STREAM_NEW*/
 
 
-#define BELLE_SIP_PARSE(object_type) \
-belle_sip_##object_type##_t* belle_sip_##object_type##_parse (const char* value) { \
+#define BELLE_PARSE(parser_name, object_type_prefix, object_type) \
+	object_type_prefix##object_type##_t* object_type_prefix##object_type##_parse (const char* value) { \
 	pANTLR3_INPUT_STREAM           input; \
 	pbelle_sip_messageLexer               lex; \
 	pANTLR3_COMMON_TOKEN_STREAM    tokens; \
-	pbelle_sip_messageParser              parser; \
-	belle_sip_##object_type##_t* l_parsed_object; \
+	p##parser_name              parser; \
+	object_type_prefix##object_type##_t* l_parsed_object; \
 	input  = ANTLR_STREAM_NEW(object_type,value,strlen(value));\
 	lex    = belle_sip_messageLexerNew                (input);\
 	tokens = antlr3CommonTokenStreamSourceNew  (ANTLR3_SIZE_HINT, TOKENSOURCE(lex));\
-	parser = belle_sip_messageParserNew               (tokens);\
+	parser = parser_name##New               (tokens);\
 	l_parsed_object = parser->object_type(parser);\
 	parser ->free(parser);\
 	tokens ->free(tokens);\
@@ -395,21 +405,23 @@ belle_sip_##object_type##_t* belle_sip_##object_type##_parse (const char* value)
 	if (l_parsed_object == NULL) belle_sip_error(#object_type" parser error for [%s]",value);\
 	return l_parsed_object;\
 }
+#define BELLE_SIP_PARSE(object_type) BELLE_PARSE(belle_sip_messageParser,belle_sip_,object_type)
 
-#define BELLE_SIP_NEW(object_type,super_type) \
-	BELLE_SIP_DECLARE_NO_IMPLEMENTED_INTERFACES(belle_sip_##object_type##_t); \
-	BELLE_SIP_INSTANCIATE_VPTR(	belle_sip_##object_type##_t\
-									, belle_sip_##super_type##_t\
-									, belle_sip_##object_type##_destroy\
-									, belle_sip_##object_type##_clone\
-									, belle_sip_##object_type##_marshal, TRUE); \
-	belle_sip_##object_type##_t* belle_sip_##object_type##_new () { \
-		belle_sip_##object_type##_t* l_object = belle_sip_object_new(belle_sip_##object_type##_t);\
-		belle_sip_##super_type##_init((belle_sip_##super_type##_t*)l_object); \
-		belle_sip_##object_type##_init((belle_sip_##object_type##_t*) l_object); \
+#define BELLE_NEW(object_type,super_type) \
+	BELLE_SIP_DECLARE_NO_IMPLEMENTED_INTERFACES(object_type##_t); \
+	BELLE_SIP_INSTANCIATE_VPTR(	object_type##_t\
+								, super_type##_t\
+								, object_type##_destroy\
+								, object_type##_clone\
+								, object_type##_marshal, TRUE); \
+		object_type##_t* object_type##_new () { \
+		object_type##_t* l_object = belle_sip_object_new(object_type##_t);\
+		super_type##_init((super_type##_t*)l_object); \
+		object_type##_init((object_type##_t*) l_object); \
 		return l_object;\
 	}
 
+#define BELLE_SIP_NEW(object_type,super_type) BELLE_NEW (belle_sip_##object_type, belle_sip_##super_type)
 
 #define BELLE_SIP_NEW_HEADER(object_type,super_type,name) BELLE_SIP_NEW_HEADER_INIT(object_type,super_type,name,header)
 #define BELLE_SIP_NEW_HEADER_INIT(object_type,super_type,name,init_type) \
@@ -426,14 +438,11 @@ belle_sip_##object_type##_t* belle_sip_##object_type##_parse (const char* value)
 		if (name) belle_sip_header_set_name(BELLE_SIP_HEADER(l_object),name);\
 		return l_object;\
 	}
-typedef struct belle_sip_param_pair_t {
+struct belle_sip_param_pair {
 	int ref;
 	char* name;
 	char* value;
-} belle_sip_param_pair_t;
-
-
-belle_sip_param_pair_t* belle_sip_param_pair_new(const char* name,const char* value);
+} ;
 
 void belle_sip_param_pair_destroy(belle_sip_param_pair_t*  pair) ;
 
@@ -445,7 +454,7 @@ belle_sip_param_pair_t* belle_sip_param_pair_ref(belle_sip_param_pair_t* obj);
 void belle_sip_param_pair_unref(belle_sip_param_pair_t* obj);
 
 
-void belle_sip_header_address_set_quoted_displayname(belle_sip_header_address_t* address,const char* value);
+
 
 /*calss header*/
 struct _belle_sip_header {
@@ -455,8 +464,7 @@ struct _belle_sip_header {
 	char *unparsed_value;
 };
 
-void belle_sip_header_set_next(belle_sip_header_t* header,belle_sip_header_t* next);
-BELLESIP_INTERNAL_EXPORT belle_sip_header_t* belle_sip_header_get_next(const belle_sip_header_t* headers);
+
 void belle_sip_response_fill_for_dialog(belle_sip_response_t *obj, belle_sip_request_t *req);
 void belle_sip_util_copy_headers(belle_sip_message_t *orig, belle_sip_message_t *dest, const char*header, int multiple);
 
@@ -508,6 +516,7 @@ struct belle_sip_stack{
 
 belle_sip_hop_t* belle_sip_hop_new(const char* transport, const char *cname, const char* host,int port);
 belle_sip_hop_t* belle_sip_hop_new_from_uri(const belle_sip_uri_t *uri);
+belle_sip_hop_t* belle_sip_hop_new_from_generic_uri(const belle_generic_uri_t *uri);
 
 belle_sip_hop_t * belle_sip_stack_get_next_hop(belle_sip_stack_t *stack, belle_sip_request_t *req);
 const belle_sip_timer_config_t *belle_sip_stack_get_timer_config(const belle_sip_stack_t *stack);
@@ -561,6 +570,17 @@ typedef struct listener_ctx{
 #define BELLE_SIP_PROVIDER_INVOKE_LISTENERS(listeners,callback,event) \
 	BELLE_SIP_INVOKE_LISTENERS_ARG((listeners),belle_sip_listener_t,callback,(event))
 
+/*
+ * http provider
+ */
+belle_http_provider_t *belle_http_provider_new(belle_sip_stack_t *s, const char *bind_ip);
+
+
+/*
+ * SIP and http messages
+**/
+
+void belle_sip_message_init(belle_sip_message_t *message);
 
 struct _belle_sip_message {
 	belle_sip_object_t base;
@@ -578,6 +598,21 @@ struct _belle_sip_request {
 	unsigned char dialog_queued;
 };
 
+
+/** HTTP request**/
+
+struct belle_http_request{
+	belle_sip_message_t base;
+	belle_generic_uri_t *req_uri;
+	char* method;
+	belle_http_request_listener_t *listener;
+	belle_generic_uri_t *orig_uri;/*original uri before removing host and user/passwd*/
+	belle_http_response_t *response;
+	int auth_attempt_count;
+};
+
+void belle_http_request_set_listener(belle_http_request_t *req, belle_http_request_listener_t *l);
+void belle_http_request_set_response(belle_http_request_t *req, belle_http_response_t *resp);
 /*
  belle_sip_transaction_t
 */
@@ -772,15 +807,6 @@ void belle_sip_dialog_queue_client_transaction(belle_sip_dialog_t *dialog, belle
 */
 belle_sip_hop_t* belle_sip_response_get_return_hop(belle_sip_response_t *msg);
 
-#define IS_TOKEN(token) \
-		(INPUT->toStringTT(INPUT,LT(1),LT(strlen(#token)))->chars ?\
-		strcasecmp(#token,(const char*)(INPUT->toStringTT(INPUT,LT(1),LT(strlen(#token)))->chars)) == 0:0)
-char* _belle_sip_str_dup_and_unquote_string(const char* quoted_string);
-
-#define IS_HEADER_NAMED(name,compressed_name) (IS_TOKEN(compressed_name) || IS_TOKEN(name))
-
-#define STRCASECMP_HEADER_NAMED(name,compressed_name,value) \
-		(strcasecmp(compressed_name,(const char*)value) == 0 || strcasecmp(name,(const char*)value) == 0 )
 
 /*********************************************************
  * SDP
@@ -845,21 +871,21 @@ struct belle_sip_io_error_event{
 };
 
 struct belle_sip_request_event{
-	belle_sip_provider_t *source;
+	belle_sip_object_t *source;
 	belle_sip_server_transaction_t *server_transaction;
 	belle_sip_dialog_t *dialog;
 	belle_sip_request_t *request;
 };
 
 struct belle_sip_response_event{
-	belle_sip_provider_t *source;
+	belle_sip_object_t *source;
 	belle_sip_client_transaction_t *client_transaction;
 	belle_sip_dialog_t *dialog;
 	belle_sip_response_t *response;
 };
 
 struct belle_sip_timeout_event{
-	belle_sip_provider_t *source;
+	belle_sip_object_t *source;
 	belle_sip_transaction_t *transaction;
 	int is_server_transaction;
 };
@@ -871,6 +897,7 @@ struct belle_sip_transaction_terminated_event{
 };
 
 struct belle_sip_auth_event {
+	belle_sip_object_t *source;
 	belle_sip_auth_mode_t mode;
 	char* username;
 	char* userid;
@@ -881,10 +908,9 @@ struct belle_sip_auth_event {
 	char* distinguished_name;
 	belle_sip_certificates_chain_t * cert;
 	belle_sip_signing_key_t* key;
-
 };
 
-belle_sip_auth_event_t* belle_sip_auth_event_create(const char* realm,const belle_sip_header_from_t * from);
+belle_sip_auth_event_t* belle_sip_auth_event_create(belle_sip_object_t *source, const char* realm,const belle_sip_header_from_t * from);
 
 void belle_sip_auth_event_set_distinguished_name(belle_sip_auth_event_t* event,const char* value);
 
@@ -902,9 +928,13 @@ BELLESIP_INTERNAL_EXPORT	char* belle_sip_uri_to_escaped_username(const char* buf
 BELLESIP_INTERNAL_EXPORT	char* belle_sip_uri_to_escaped_userpasswd(const char* buff) ;
 BELLESIP_INTERNAL_EXPORT	char* belle_sip_uri_to_escaped_parameter(const char* buff) ;
 BELLESIP_INTERNAL_EXPORT	char* belle_sip_uri_to_escaped_header(const char* buff) ;
-BELLESIP_INTERNAL_EXPORT	char* belle_sip_to_unescaped_string(const char* buff) ;
 
 
+/*(uri RFC 2396)*/
+
+BELLESIP_INTERNAL_EXPORT char* belle_generic_uri_to_escaped_query(const char* buff);
+BELLESIP_INTERNAL_EXPORT char* belle_generic_uri_to_escaped_path(const char* buff);
+
 #define BELLE_SIP_SOCKET_TIMEOUT 30000
 
 #define BELLE_SIP_BRANCH_ID_LENGTH 10
@@ -917,4 +947,30 @@ void belle_sip_request_set_dialog(belle_sip_request_t *req, belle_sip_dialog_t *
 void belle_sip_request_set_rfc2543_branch(belle_sip_request_t *req, const char *rfc2543branch);
 void belle_sip_dialog_update_request(belle_sip_dialog_t *dialog, belle_sip_request_t *req);
 
+belle_sip_error_code belle_sip_headers_marshal(belle_sip_message_t *message, char* buff, size_t buff_size, size_t *offset);
+
+#define SET_OBJECT_PROPERTY(obj,property_name,new_value) \
+	if (new_value) belle_sip_object_ref(new_value); \
+	if (obj->property_name){ \
+		belle_sip_object_unref(obj->property_name); \
+	}\
+	obj->property_name=new_value;
+
+#include "parserutils.h"
+
+/******************************
+ *
+ * private Extension header inherit from header
+ *
+ ******************************/
+typedef struct _belle_sip_header_extension belle_sip_header_extension_t;
+
+belle_sip_header_extension_t* belle_sip_header_extension_new();
+
+belle_sip_header_extension_t* belle_sip_header_extension_parse (const char* extension) ;
+belle_sip_header_extension_t* belle_sip_header_extension_create (const char* name,const char* value);
+const char* belle_sip_header_extension_get_value(const belle_sip_header_extension_t* extension);
+void belle_sip_header_extension_set_value(belle_sip_header_extension_t* extension,const char* value);
+#define BELLE_SIP_HEADER_EXTENSION(t) BELLE_SIP_CAST(t,belle_sip_header_extension_t)
+
 #endif
