@@ -4,7 +4,7 @@
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -165,17 +165,27 @@ static struct dns_resolv_conf *resconf(belle_sip_simple_resolver_context_t *ctx)
 	
 	path=belle_sip_stack_get_dns_resolv_conf_file(ctx->base.stack);
 	if (!path){
-#ifdef _WIN32
+#if defined(USE_FIXED_NAMESERVERS)
+		error = dns_resconf_load_fixed_nameservers(ctx->resconf);
+		if (error) {
+			belle_sip_error("%s dns_resconf_load_fixed_nameservers error", __FUNCTION__);
+		}
+#elif defined(USE_STRUCT_RES_STATE_NAMESERVERS)
+		error = dns_resconf_load_struct_res_state_nameservers(ctx->resconf);
+		if (error) {
+			belle_sip_error("%s dns_resconf_load_struct_res_state_nameservers error", __FUNCTION__);
+		}
+#elif defined(_WIN32)
 		error = dns_resconf_loadwin(ctx->resconf);
 		if (error) {
 			belle_sip_error("%s dns_resconf_loadwin error", __FUNCTION__);
 		}
-#elif ANDROID
+#elif defined(ANDROID)
 		error = dns_resconf_loadandroid(ctx->resconf);
 		if (error) {
 			belle_sip_error("%s dns_resconf_loadandroid error", __FUNCTION__);
 		}
-#elif HAVE_RESINIT
+#elif defined(HAVE_RESINIT)
 /*#elif HAVE_RESINIT && TARGET_OS_IPHONE*/
 		error = dns_resconf_loadfromresolv(ctx->resconf);
 		if (error) {
@@ -213,7 +223,8 @@ static struct dns_resolv_conf *resconf(belle_sip_simple_resolver_context_t *ctx)
 		for(i=0;i<sizeof(ctx->resconf->nameserver)/sizeof(ctx->resconf->nameserver[0]);++i){
 			struct sockaddr *ns_addr=(struct sockaddr*)&ctx->resconf->nameserver[i];
 			if (ns_addr->sa_family==AF_UNSPEC) break;
-			getnameinfo(ns_addr,sizeof(struct sockaddr_storage),ip,sizeof(ip),serv,sizeof(serv),NI_NUMERICHOST|NI_NUMERICSERV);
+			getnameinfo(ns_addr,ns_addr->sa_family==AF_INET6 ? sizeof(struct sockaddr_in6) : sizeof(struct sockaddr)
+					,ip,sizeof(ip),serv,sizeof(serv),NI_NUMERICHOST|NI_NUMERICSERV);
 			belle_sip_message("\t%s",ip);
 			if (ns_addr->sa_family==AF_INET6) using_ipv6=TRUE;
 		}
@@ -484,7 +495,10 @@ struct addrinfo * belle_sip_ip_address_to_addrinfo(int family, const char *ipadd
 		hints.ai_flags|=AI_V4MAPPED;
 	}
 	err=getaddrinfo(ipaddress,serv,&hints,&res);
+
 	if (err!=0){
+		if (err!=EAI_NONAME)
+			belle_sip_error("belle_sip_ip_address_to_addrinfo(): getaddrinfo() error: %s",gai_strerror(err));
 		return NULL;
 	}
 	return res;
@@ -781,6 +795,7 @@ static void notify_dual_results(belle_sip_dual_resolver_context_t *ctx){
 		results=ai_list_append(results,ctx->a_results);
 		ctx->a_results=NULL;
 		ctx->aaaa_results=NULL;
+		ctx->base.done=TRUE;
 		ctx->cb(ctx->cb_data,ctx->name,results);
 		belle_sip_object_unref(ctx);
 	}
@@ -804,20 +819,21 @@ static belle_sip_resolver_context_t * belle_sip_stack_resolve_dual(belle_sip_sta
 	/* Then perform asynchronous DNS A or AAAA query */
 	belle_sip_dual_resolver_context_t *ctx = belle_sip_object_new(belle_sip_dual_resolver_context_t);
 	belle_sip_resolver_context_init((belle_sip_resolver_context_t*)ctx,stack);
-	
+	belle_sip_object_ref(ctx);/*we don't want the object to be destroyed until the end of this function*/
 	ctx->cb_data = data;
 	ctx->cb = cb;
 	ctx->name = belle_sip_strdup(name);
+	/*take a ref for the entire duration of the DNS procedure, it will be released when it is finished*/
 	belle_sip_object_ref(ctx);
 	ctx->a_ctx=belle_sip_stack_resolve_single(stack,name,port,AF_INET, AI_V4MAPPED, on_ipv4_results,ctx);
 	if (ctx->a_ctx) belle_sip_object_ref(ctx->a_ctx);
 	ctx->aaaa_ctx=belle_sip_stack_resolve_single(stack, name, port, AF_INET6, 0, on_ipv6_results, ctx);
 	if (ctx->aaaa_ctx) belle_sip_object_ref(ctx->aaaa_ctx);
-	if (ctx->aaaa_ctx==NULL && ctx->a_ctx==NULL){
-		/*all results found synchronously*/
+	if (ctx->base.done){
+		/*all results were found synchronously*/
 		belle_sip_object_unref(ctx);
 		ctx=NULL;
-	}
+	}else belle_sip_object_unref(ctx);
 	return BELLE_SIP_RESOLVER_CONTEXT(ctx);
 }
 
@@ -887,14 +903,14 @@ void belle_sip_get_src_addr_for(const struct sockaddr *dest, socklen_t destlen,
 	return;
 fail:
 	{
-		struct addrinfo hints={0},*res=NULL;
-		int err;
-		hints.ai_family=af_type;
-		err=getaddrinfo(af_type==AF_INET ? "0.0.0.0" : "::0","0",&hints,&res);
-		if (err!=0) belle_sip_fatal("belle_sip_get_src_addr_for(): getaddrinfo failed: %s",belle_sip_get_socket_error_string_from_code(err));
-		memcpy(src,res->ai_addr,MIN((size_t)*srclen,res->ai_addrlen));
-		*srclen=res->ai_addrlen;
-		freeaddrinfo(res);
+		struct addrinfo *res = belle_sip_ip_address_to_addrinfo(af_type, af_type == AF_INET ? "127.0.0.1" : "::1", local_port);
+		if (res != NULL) {
+			memcpy(src,res->ai_addr,MIN((size_t)*srclen,res->ai_addrlen));
+			*srclen=res->ai_addrlen;
+			freeaddrinfo(res);
+		} else {
+			belle_sip_fatal("belle_sip_get_src_addr_for(): belle_sip_ip_address_to_addrinfo() failed");
+		}
 	}
 	if (sock!=(belle_sip_socket_t)-1) close_socket(sock);
 }
