@@ -4,7 +4,7 @@
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -16,13 +16,13 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include "belle-sip/uri.h"
+#include "belle-sip/sip-uri.h"
 #include "belle-sip/parameters.h"
 #include <stdlib.h>
 #include <string.h>
 #include <stdarg.h>
-#include "belle_sip_messageParser.h"
-#include "belle_sip_messageLexer.h"
+#include "grammars/belle_sip_messageParser.h"
+#include "grammars/belle_sip_messageLexer.h"
 #include "belle_sip_internal.h"
 #include "listeningpoint_internal.h"
 
@@ -144,10 +144,10 @@ belle_sip_error_code belle_sip_uri_marshal(const belle_sip_uri_t* uri, char* buf
 			belle_sip_param_pair_t* container = list->data;
 			if (list == encheaders) {
 				//first case
-				error=belle_sip_snprintf(buff,buff_size,offset,"?%s=%s",container->name,container->value);
+				error=belle_sip_snprintf(buff,buff_size,offset,"?%s=%s",container->name,container->value?container->value:"");
 			} else {
 				//subsequent headers
-				error=belle_sip_snprintf(buff,buff_size,offset,"&%s=%s",container->name,container->value);
+				error=belle_sip_snprintf(buff,buff_size,offset,"&%s=%s",container->name,container->value?container->value:"");
 			}
 			if (error!=BELLE_SIP_OK) break;
 		}
