@@ -4,7 +4,7 @@
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -18,6 +18,7 @@
 
 
 #include "belle_sip_internal.h"
+#include <limits.h>
 
 static void channel_prepare_continue(belle_sip_channel_t *obj);
 static void channel_process_queue(belle_sip_channel_t *obj);
@@ -167,10 +168,11 @@ static int get_message_start_pos(char *buff, size_t bufflen) {
 		saved_char1=buff[saved_char1_index]; /*make sure buff is null terminated*/
 		buff[saved_char1_index]='\0';
 		res=sscanf(buff+i,"SIP/2.0 %d ",&status_code);
+		if (res!=1) res=sscanf(buff+i,"HTTP/1.%*i %d ",&status_code); /*might be HTTP ?*/
 		if (res!=1) {
 			res= sscanf(buff+i,"%16s %*s %9s\r\n",method,sip_version)==2
 					&& is_token(method,sizeof(method))
-					&& strcmp("SIP/2.0",sip_version)==0 ;
+					&& (strcmp("SIP/2.0",sip_version)==0 || strncmp("HTTP/1.",sip_version,strlen("HTTP/1."))==0) ;
 		}
 		buff[saved_char1_index]=saved_char1;
 		if (res==1) return i;
@@ -193,6 +195,12 @@ static void belle_sip_channel_input_stream_reset(belle_sip_channel_input_stream_
 	}
 	input_stream->state=WAITING_MESSAGE_START;
 	input_stream->msg=NULL;
+	input_stream->chuncked_mode=FALSE;
+	input_stream->content_length=-1;
+	if (input_stream->body){
+		belle_sip_free(input_stream->body);
+		input_stream->body=NULL;
+	}
 }
 
 static size_t belle_sip_channel_input_stream_get_buff_length(belle_sip_channel_input_stream_t* input_stream) {
@@ -256,11 +264,117 @@ static void belle_sip_channel_message_ready(belle_sip_channel_t *obj){
 	belle_sip_channel_input_stream_reset(&obj->input_stream);
 }
 
-void belle_sip_channel_parse_stream(belle_sip_channel_t *obj){
+/*returns TRUE if a body is expected, and initialize a few things in the input stream context*/
+static int check_body(belle_sip_channel_t *obj){
+	belle_sip_message_t *msg=obj->input_stream.msg;
+	belle_sip_header_content_length_t* content_length_header = belle_sip_message_get_header_by_type(msg,belle_sip_header_content_length_t);
+	obj->input_stream.content_length= content_length_header ? belle_sip_header_content_length_get_content_length(content_length_header) : 0;
+	
+	if (BELLE_SIP_OBJECT_IS_INSTANCE_OF(msg,belle_sip_response_t) || BELLE_SIP_OBJECT_IS_INSTANCE_OF(msg,belle_sip_request_t)){
+		return obj->input_stream.content_length>0;
+	}else{/*http*/
+		if (belle_sip_message_get_header_by_type(msg, belle_sip_header_content_type_t)!=NULL){
+			belle_sip_header_t *transfer_encoding=belle_sip_message_get_header(msg,"Transfer-Encoding");
+			
+			if (transfer_encoding){
+				const char *value=belle_sip_header_get_unparsed_value(transfer_encoding);
+				if (strstr(value,"chunked")!=0){
+					obj->input_stream.chuncked_mode=1;
+					obj->input_stream.content_length=0;
+					obj->input_stream.chunk_size=-1;
+				}
+			}
+			return TRUE;
+		}
+	}
+	return FALSE;
+}
+
+static int acquire_body_simple(belle_sip_channel_t *obj, int end_of_stream){
+	int content_length=obj->input_stream.content_length;
+	
+	if (end_of_stream && content_length==-1){
+		content_length=obj->input_stream.write_ptr-obj->input_stream.read_ptr;
+	}
+	if (content_length!=-1 && content_length <= obj->input_stream.write_ptr-obj->input_stream.read_ptr){
+		/*great body completed
+		belle_sip_message("channel [%p] read [%i] bytes of body from %s:%i\n%s"	,obj
+			,content_length
+			,obj->peer_name
+			,obj->peer_port
+			,obj->input_stream.read_ptr);*/
+		belle_sip_message_set_body(obj->input_stream.msg,obj->input_stream.read_ptr,content_length);
+		obj->input_stream.read_ptr+=content_length;
+		belle_sip_channel_message_ready(obj);
+		return BELLE_SIP_CONTINUE;
+	}else{
+		/*body is not finished, we need more data*/
+		return BELLE_SIP_STOP;
+	}
+}
+
+static int acquire_chuncked_body(belle_sip_channel_t *obj, int end_of_stream){
+	belle_sip_channel_input_stream_t *st=&obj->input_stream;
+	do{
+		if (st->chunk_size==-1){
+			char *tmp;
+			belle_sip_message("seeing: %s",st->read_ptr);
+			while ( (tmp=strstr(st->read_ptr,"\r\n"))==st->read_ptr){/*skip \r\n*/
+				st->read_ptr+=2;
+			}
+			
+			if (tmp!=NULL){
+				/*the chunk length is there*/
+				long chunksize=strtol(st->read_ptr,NULL,16);
+				if (chunksize>=0 && chunksize!=LONG_MAX){
+					if (chunksize==0){
+						belle_sip_message("Got end of chunked body");
+						belle_sip_message_assign_body(obj->input_stream.msg,(char*)st->body,st->content_length);
+						st->body=NULL;
+						st->read_ptr=tmp+4; /*last chunk indicator finishes with two \r\n*/
+						if (st->read_ptr>st->write_ptr) st->read_ptr=st->write_ptr;
+						belle_sip_channel_message_ready(obj);
+						return BELLE_SIP_CONTINUE;
+					}else{
+						belle_sip_message("Getting chunk of %i bytes",(int)chunksize);
+						st->chunk_size=chunksize;
+						st->read_ptr=tmp+2;
+					}
+				}else{
+					belle_sip_error("Chunk parse error");
+					belle_sip_channel_input_stream_reset(st);
+					return BELLE_SIP_CONTINUE;
+				}
+			}else{
+				/*need more data*/
+				return BELLE_SIP_STOP;
+			}
+		}
+		if (st->chunk_size<=st->write_ptr-st->read_ptr){
+			/*we have a chunk completed*/
+			int prev_ctlen=st->content_length;
+			st->content_length+=st->chunk_size;
+			st->body=belle_sip_realloc(st->body,st->content_length);
+			memcpy(st->body+prev_ctlen,st->read_ptr,st->chunk_size);
+			st->read_ptr+=st->chunk_size;
+			belle_sip_message("Chunk of [%i] bytes completed",st->chunk_size);
+			st->chunk_size=-1;/*wait for next chunk indicator*/
+		}else{
+			/*need more data*/
+			return BELLE_SIP_STOP;
+		}
+	}while(1);
+}
+
+static int acquire_body(belle_sip_channel_t *obj, int end_of_stream){
+	if (obj->input_stream.chuncked_mode)
+		return acquire_chuncked_body(obj,end_of_stream);
+	else return acquire_body_simple(obj,end_of_stream);
+}
+
+void belle_sip_channel_parse_stream(belle_sip_channel_t *obj, int end_of_stream){
 	int offset;
 	size_t read_size=0;
-	belle_sip_header_content_length_t* content_length_header;
-	int content_length;
 	int num;
 	
 	while ((num=(obj->input_stream.write_ptr-obj->input_stream.read_ptr))>0){
@@ -304,9 +418,8 @@ void belle_sip_channel_parse_stream(belle_sip_channel_t *obj){
 					belle_sip_object_ref(obj->input_stream.msg);
 					if (belle_sip_message_is_request(obj->input_stream.msg)) fix_incoming_via(BELLE_SIP_REQUEST(obj->input_stream.msg),obj->current_peer);
 					/*check for body*/
-					if ((content_length_header = (belle_sip_header_content_length_t*)belle_sip_message_get_header(obj->input_stream.msg,BELLE_SIP_CONTENT_LENGTH)) != NULL
-							&& belle_sip_header_content_length_get_content_length(content_length_header)>0) {
-
+					
+					if (check_body(obj)){
 						obj->input_stream.state=BODY_AQUISITION;
 					} else {
 						/*no body*/
@@ -325,23 +438,11 @@ void belle_sip_channel_parse_stream(belle_sip_channel_t *obj){
 		}
 
 		if (obj->input_stream.state==BODY_AQUISITION) {
-			content_length=belle_sip_header_content_length_get_content_length((belle_sip_header_content_length_t*)belle_sip_message_get_header(obj->input_stream.msg,BELLE_SIP_CONTENT_LENGTH));
-			if (content_length <= obj->input_stream.write_ptr-obj->input_stream.read_ptr) {
-				/*great body completed
-				belle_sip_message("channel [%p] read [%i] bytes of body from %s:%i\n%s"	,obj
-					,content_length
-					,obj->peer_name
-					,obj->peer_port
-					,obj->input_stream.read_ptr);*/
-				belle_sip_message_set_body(obj->input_stream.msg,obj->input_stream.read_ptr,content_length);
-				obj->input_stream.read_ptr+=content_length;
-				belle_sip_channel_message_ready(obj);
-			}else{
-				/*body is not finished, we need more data*/
-				break;
-			}
+			if (acquire_body(obj,end_of_stream)==BELLE_SIP_STOP) break;
 		}
 	}
+	if (obj->incoming_messages)
+		BELLE_SIP_INVOKE_LISTENERS_ARG1_ARG2(obj->listeners,belle_sip_channel_listener_t,on_event,obj,BELLE_SIP_EVENT_READ/*always a read event*/);
 }
 
 int belle_sip_channel_process_data(belle_sip_channel_t *obj,unsigned int revents){
@@ -371,14 +472,16 @@ int belle_sip_channel_process_data(belle_sip_channel_t *obj,unsigned int revents
 					obj->peer_name,
 					obj->peer_port,
 					begin);
-		belle_sip_channel_parse_stream(obj);
-		if (obj->incoming_messages)
-			BELLE_SIP_INVOKE_LISTENERS_ARG1_ARG2(obj->listeners,belle_sip_channel_listener_t,on_event,obj,BELLE_SIP_EVENT_READ/*always a read event*/);
-		
+		belle_sip_channel_parse_stream(obj,FALSE);
 	} else if (num == 0) {
+		/*before closing the channel, check if there was a pending message to receive, whose body acquisition is to be finished.*/
+		belle_sip_channel_parse_stream(obj,TRUE);
 		channel_set_state(obj,BELLE_SIP_CHANNEL_DISCONNECTED);
 		return BELLE_SIP_STOP;
-	} else {
+	} else if ( belle_sip_error_code_is_would_block(-num)){
+		belle_sip_message("EWOULDBLOCK");
+		return BELLE_SIP_CONTINUE;
+	}else{
 		belle_sip_error("Receive error on channel [%p]",obj);
 		channel_set_state(obj,BELLE_SIP_CHANNEL_ERROR);
 		return BELLE_SIP_STOP;
@@ -413,17 +516,29 @@ static void update_inactivity_timer(belle_sip_channel_t *obj, int from_recv){
 		obj->last_recv_time=belle_sip_time_ms();
 }
 
+/*constructor for channels creating an outgoing connection
+ * bindip local ip address to bind on, typically 0.0.0.0 or ::0
+ * locaport locaport to use for binding, can be set to 0 if port doesn't matter
+ * peer_cname canonical name of remote host, used for TLS verification
+ * peername peer's hostname, either ip address or DNS name
+ * pee_port peer's port to connect to.
+ */
 void belle_sip_channel_init(belle_sip_channel_t *obj, belle_sip_stack_t *stack,const char *bindip,int localport,const char *peer_cname, const char *peername, int peer_port){
 	/*to initialize our base class:*/
 	belle_sip_channel_set_socket(obj,-1,NULL);
 	
 	/*then initialize members*/
+	obj->ai_family=AF_INET;
 	obj->peer_cname=peer_cname ? belle_sip_strdup(peer_cname) : NULL;
 	obj->peer_name=belle_sip_strdup(peername);
 	obj->peer_port=peer_port;
 	obj->stack=stack;
-	if (bindip && strcmp(bindip,"::0")!=0 && strcmp(bindip,"0.0.0.0")!=0)
-		obj->local_ip=belle_sip_strdup(bindip);
+	if (bindip){
+		if (strcmp(bindip,"::0")!=0 && strcmp(bindip,"0.0.0.0")!=0)
+			obj->local_ip=belle_sip_strdup(bindip);
+		if (strchr(bindip,':')!=NULL)
+			obj->ai_family=AF_INET6;
+	}
 	obj->local_port=localport;
 	obj->simulated_recv_return=1;/*not set*/
 	if (peername){
@@ -436,6 +551,7 @@ void belle_sip_channel_init(belle_sip_channel_t *obj, belle_sip_stack_t *stack,c
 	update_inactivity_timer(obj,FALSE);
 }
 
+/*constructor for channels created by incoming connections*/
 void belle_sip_channel_init_with_addr(belle_sip_channel_t *obj, belle_sip_stack_t *stack, const struct sockaddr *peer_addr, socklen_t addrlen){
 	char remoteip[64];
 	struct addrinfo ai;
@@ -448,6 +564,7 @@ void belle_sip_channel_init_with_addr(belle_sip_channel_t *obj, belle_sip_stack_
 	belle_sip_addrinfo_to_ip(&ai,remoteip,sizeof(remoteip),&peer_port);
 	belle_sip_channel_init(obj,stack,NULL,0,NULL,remoteip,peer_port);
 	obj->peer_list=obj->current_peer=belle_sip_ip_address_to_addrinfo(ai.ai_family, obj->peer_name,obj->peer_port);
+	obj->ai_family=ai.ai_family;
 }
 
 void belle_sip_channel_set_socket(belle_sip_channel_t *obj, belle_sip_socket_t sock, belle_sip_source_func_t datafunc){
@@ -550,6 +667,11 @@ static void channel_invoke_state_listener_defered(belle_sip_channel_t *obj){
 	belle_sip_object_unref(obj);
 }
 
+static void channel_connect_next(belle_sip_channel_t *obj){
+	belle_sip_channel_connect(obj);
+	belle_sip_object_unref(obj);
+}
+
 static void belle_sip_channel_handle_error(belle_sip_channel_t *obj){
 	if (obj->state!=BELLE_SIP_CHANNEL_READY){
 		/* Previous connection attempts were failed (channel could not get ready).*/
@@ -558,7 +680,7 @@ static void belle_sip_channel_handle_error(belle_sip_channel_t *obj){
 			obj->current_peer=obj->current_peer->ai_next;
 			channel_set_state(obj,BELLE_SIP_CHANNEL_RETRY);
 			belle_sip_channel_close(obj);
-			belle_sip_channel_connect(obj);
+			belle_sip_main_loop_do_later(obj->stack->ml,(belle_sip_callback_t)channel_connect_next,belle_sip_object_ref(obj));
 			return;
 		}/*else we have already tried all the ip addresses, so give up and notify the error*/
 	}/*else the channel was previously working good with the current ip address but now fails, so let's notify the error*/
@@ -568,8 +690,7 @@ static void belle_sip_channel_handle_error(belle_sip_channel_t *obj){
 	* it is safer to invoke the listener outside the current call stack.
 	* Indeed the channel encounters network errors while being called for transmiting by a transaction.
 	*/
-	belle_sip_object_ref(obj);
-	belle_sip_main_loop_do_later(obj->stack->ml,(belle_sip_callback_t)channel_invoke_state_listener_defered,obj);
+	belle_sip_main_loop_do_later(obj->stack->ml,(belle_sip_callback_t)channel_invoke_state_listener_defered,belle_sip_object_ref(obj));
 }
 
 int belle_sip_channel_notify_timeout(belle_sip_channel_t *obj){
@@ -719,10 +840,11 @@ void belle_sip_channel_set_ready(belle_sip_channel_t *obj, const struct sockaddr
 	if (obj->local_ip==NULL){
 		struct sockaddr_storage saddr;
 		socklen_t slen2=sizeof(saddr);
+		int err;
 		
 		belle_sip_address_remove_v4_mapping(addr,(struct sockaddr*) &saddr,&slen2);
 		
-		int err=getnameinfo((struct sockaddr*)&saddr,slen,name,sizeof(name),serv,sizeof(serv),NI_NUMERICHOST|NI_NUMERICSERV);
+		err=getnameinfo((struct sockaddr*)&saddr,slen,name,sizeof(name),serv,sizeof(serv),NI_NUMERICHOST|NI_NUMERICSERV);
 		if (err!=0){
 			belle_sip_error("belle_sip_channel_set_ready(): getnameinfo() failed: %s",gai_strerror(err));
 		}else{
@@ -753,10 +875,10 @@ static void channel_res_done(void *data, const char *name, struct addrinfo *ai_l
 
 void belle_sip_channel_resolve(belle_sip_channel_t *obj){
 	channel_set_state(obj,BELLE_SIP_CHANNEL_RES_IN_PROGRESS);
-	if (belle_sip_stack_dns_srv_enabled(obj->stack))
-		obj->resolver_ctx=belle_sip_stack_resolve(obj->stack, belle_sip_channel_get_transport_name_lower_case(obj), obj->peer_name, obj->peer_port, obj->lp->ai_family, channel_res_done, obj);
+	if (belle_sip_stack_dns_srv_enabled(obj->stack) && obj->lp!=NULL)
+		obj->resolver_ctx=belle_sip_stack_resolve(obj->stack, belle_sip_channel_get_transport_name_lower_case(obj), obj->peer_name, obj->peer_port, obj->ai_family, channel_res_done, obj);
 	else
-		obj->resolver_ctx=belle_sip_stack_resolve_a(obj->stack, obj->peer_name, obj->peer_port, obj->lp->ai_family, channel_res_done, obj);
+		obj->resolver_ctx=belle_sip_stack_resolve_a(obj->stack, obj->peer_name, obj->peer_port, obj->ai_family, channel_res_done, obj);
 	if (obj->resolver_ctx){
 		belle_sip_object_ref(obj->resolver_ctx);
 	}
@@ -793,4 +915,36 @@ void belle_sip_channel_force_close(belle_sip_channel_t *obj){
 	channel_set_state(obj,BELLE_SIP_CHANNEL_DISCONNECTED);
 }
 
+belle_sip_channel_t *belle_sip_channel_find_from_list_with_addrinfo(belle_sip_list_t *l, const belle_sip_hop_t *hop, const struct addrinfo *addr){
+	belle_sip_list_t *elem;
+	belle_sip_channel_t *chan;
+	
+	for(elem=l;elem!=NULL;elem=elem->next){
+		chan=(belle_sip_channel_t*)elem->data;
+		if (!chan->about_to_be_closed && belle_sip_channel_matches(chan,hop,addr)){
+			return chan;
+		}
+	}
+	return NULL;
+}
+/* search a matching channel from a list according to supplied hop. The ai_family tells which address family is supported by the list of channels*/
+belle_sip_channel_t *belle_sip_channel_find_from_list(belle_sip_list_t *l, int ai_family, const belle_sip_hop_t *hop){
+	struct addrinfo *res=NULL;
+	struct addrinfo hints={0};
+	char portstr[20];
+	belle_sip_channel_t *chan;
+
+	hints.ai_family=ai_family;
+	hints.ai_flags=AI_NUMERICHOST|AI_NUMERICSERV;
+	hints.ai_socktype=SOCK_STREAM; // needed on some platforms that return an error otherwise (QNX)
+	if (ai_family==AF_INET6) hints.ai_flags=AI_V4MAPPED;
+	
+	snprintf(portstr,sizeof(portstr),"%i",hop->port);
+	getaddrinfo(hop->host,portstr,&hints,&res);
+	chan=belle_sip_channel_find_from_list_with_addrinfo(l,hop,res);
+	if (res) freeaddrinfo(res);
+	return chan;
+}
+
+
 
