@@ -4,7 +4,7 @@
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -69,6 +69,10 @@ typedef struct belle_sip_channel_input_stream{
 	char* read_ptr;
 	char* write_ptr;
 	belle_sip_message_t *msg;
+	unsigned char *body; /*used when receiving a chunked body*/
+	int content_length;
+	int chuncked_mode;
+	int chunk_size;
 }belle_sip_channel_input_stream_t;
 
 typedef struct belle_sip_stream_channel belle_sip_stream_channel_t;
@@ -76,10 +80,11 @@ typedef struct belle_sip_tls_channel belle_sip_tls_channel_t;
 
 struct belle_sip_channel{
 	belle_sip_source_t base;
-	belle_sip_listening_point_t *lp; /*the listening point that owns this channel*/
 	belle_sip_stack_t *stack;
+	belle_sip_listening_point_t *lp; /*the listening point that owns this channel*/
 	belle_sip_channel_state_t state;
 	belle_sip_list_t *listeners;
+	int ai_family;
 	char *peer_cname;
 	char *peer_name;
 	int peer_port;
@@ -99,6 +104,7 @@ struct belle_sip_channel{
 	unsigned char force_close; /* when channel is intentionnaly disconnected, in order to prevent looping notifications*/
 	unsigned char learnt_ip_port;
 	unsigned char has_name; /*set when the name of the peer is known, which is not the case of inboud connections*/
+	unsigned char about_to_be_closed;
 };
 
 #define BELLE_SIP_CHANNEL(obj)		BELLE_SIP_CAST(obj,belle_sip_channel_t)
@@ -133,7 +139,7 @@ void belle_sip_channel_init_with_addr(belle_sip_channel_t *obj, belle_sip_stack_
 void belle_sip_channel_set_socket(belle_sip_channel_t *obj, belle_sip_socket_t sock, belle_sip_source_func_t datafunc);
 /*end of channel implementations*/
 /**
- * pickup last received message. This method take the ownership of the message.
+ * Get a received message from the receive queue. This caller takes the ownership of the message.
  */
 belle_sip_message_t* belle_sip_channel_pick_message(belle_sip_channel_t *obj);
 
@@ -182,6 +188,15 @@ BELLE_SIP_DECLARE_CUSTOM_VPTR_END
 void belle_sip_tls_channel_set_client_certificates_chain(belle_sip_tls_channel_t *obj, belle_sip_certificates_chain_t* cert_chain);
 void belle_sip_tls_channel_set_client_certificate_key(belle_sip_tls_channel_t *obj, belle_sip_signing_key_t* key);
 
+belle_sip_channel_t *belle_sip_channel_find_from_list_with_addrinfo(belle_sip_list_t *l, const belle_sip_hop_t *hop, const struct addrinfo *addr);
+belle_sip_channel_t *belle_sip_channel_find_from_list(belle_sip_list_t *l, int ai_family, const belle_sip_hop_t *hop);
+
 #define BELLE_SIP_TLS_CHANNEL(obj)		BELLE_SIP_CAST(obj,belle_sip_tls_channel_t)
 
+struct belle_tls_verify_policy{
+	belle_sip_object_t base;
+	char *root_ca;
+	int exception_flags;
+};
+
 #endif
