@@ -4,7 +4,7 @@
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -460,8 +460,8 @@ belle_sip_dialog_t *belle_sip_dialog_new(belle_sip_transaction_t *t){
 	belle_sip_message("New %s dialog [%p] , local tag [%s], remote tag [%s]"
 			,obj->is_server?"server":"client"
 			,obj
-			,obj->local_tag
-			,obj->remote_tag);
+			,obj->local_tag?obj->local_tag:""
+			,obj->remote_tag?obj->remote_tag:"");
 	set_state(obj,BELLE_SIP_DIALOG_NULL);
 	return obj;
 }
@@ -529,7 +529,7 @@ static belle_sip_request_t *create_request(belle_sip_dialog_t *obj, const char *
 	return req;
 }
 
-BELLESIP_EXPORT belle_sip_request_t * belle_sip_dialog_create_queued_request(belle_sip_dialog_t *obj, const char *method){
+belle_sip_request_t * belle_sip_dialog_create_queued_request(belle_sip_dialog_t *obj, const char *method){
 	belle_sip_request_t *req;
 	
 	if (strcmp(method,"INVITE")==0 || strcmp(method,"SUBSCRIBE")==0){
