@@ -4,7 +4,7 @@
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -15,8 +15,9 @@
     You should have received a copy of the GNU General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
-#include "belle_sip_messageParser.h"
-#include "belle_sip_messageLexer.h"
+#include "grammars/belle_sip_messageLexer.h"
+#include "grammars/belle_sip_messageParser.h"
+
 #include "belle_sip_internal.h"
 
 
@@ -25,9 +26,62 @@ typedef struct _headers_container {
 	belle_sip_list_t* header_list;
 } headers_container_t;
 
+/*reference is 
+ * http://www.iana.org/assignments/sip-parameters/sip-parameters.xhtml#sip-parameters-2
+ */
+static const char * expand_name(const char *name){
+	const char *full_name=NULL;
+	
+	if (strlen(name)>1) return name;
+	switch(name[0]){
+		case 'a':
+			full_name="Accept-Contact";
+			break;
+		case 'u':
+			full_name="Allow-Events";
+			break;
+		case 'e':
+			full_name="Content-Encoding";
+			break;
+		case 'o':
+			full_name="Event";
+			break;
+		case 'y':
+			full_name="Identity";
+			break;
+		case 'n':
+			full_name="Identity-Info";
+			break;
+		case 'r':
+			full_name="Refer-To";
+			break;
+		case 'b':
+			full_name="Referred-By";
+			break;
+		case 'j':
+			full_name="Reject-Contact";
+			break;
+		case 'd':
+			full_name="Request-Disposition";
+			break;
+		case 'x':
+			full_name="Session-Expires";
+			break;
+		case 's':
+			full_name="Subject";
+			break;
+		case 'k':
+			full_name="Supported";
+			break;
+		default:
+			full_name=name;
+	}
+	return full_name;
+}
+
 static headers_container_t* belle_sip_message_headers_container_new(const char* name) {
 	headers_container_t* headers_container = belle_sip_new0(headers_container_t);
-	headers_container->name= belle_sip_strdup(name);
+	headers_container->name = belle_sip_strdup(expand_name(name));
 	return  headers_container;
 }
 
@@ -95,7 +149,7 @@ static int belle_sip_headers_container_comp_func(const headers_container_t *a, c
 	return strcasecmp(a->name,b);
 }
 
-static void belle_sip_message_init(belle_sip_message_t *message){
+void belle_sip_message_init(belle_sip_message_t *message){
 	
 }
 
@@ -369,6 +423,10 @@ const char* belle_sip_message_get_body(belle_sip_message_t *msg) {
 	return msg->body;
 }
 
+unsigned int belle_sip_message_get_body_size(const belle_sip_message_t *msg){
+	return msg->body_length;
+}
+
 void belle_sip_message_set_body(belle_sip_message_t *msg, const char* body, unsigned int size) {
 	if (msg->body) {
 		belle_sip_free(msg->body);
@@ -486,7 +544,7 @@ belle_sip_error_code belle_sip_response_marshal(belle_sip_response_t *resp, char
 													,offset
 													,"SIP/2.0 %i %s\r\n"
 													,belle_sip_response_get_status_code(resp)
-													,belle_sip_response_get_reason_phrase(resp));
+													,belle_sip_response_get_reason_phrase(resp)?belle_sip_response_get_reason_phrase(resp):"");
 	if (error!=BELLE_SIP_OK) return error;
 	error=belle_sip_headers_marshal(BELLE_SIP_MESSAGE(resp),buff,buff_size,offset);
 	if (error!=BELLE_SIP_OK) return error;
