@@ -4,7 +4,7 @@
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -115,7 +115,7 @@ static void belle_sip_provider_dispatch_request(belle_sip_provider_t* prov, bell
 			belle_sip_server_transaction_send_response(tr,belle_sip_response_create_from_request(req,480));
 			return;
 		} else {
-			ev.source=prov;
+			ev.source=(belle_sip_object_t*)prov;
 			ev.server_transaction=NULL;
 			ev.request=req;
 			BELLE_SIP_PROVIDER_INVOKE_LISTENERS(prov->listeners,process_request_event,&ev);
@@ -156,7 +156,7 @@ static void belle_sip_provider_dispatch_response(belle_sip_provider_t* prov, bel
 		belle_sip_object_unref(t);
 	}else{
 		belle_sip_response_event_t event;
-		event.source=prov;
+		event.source=(belle_sip_object_t*)prov;
 		event.client_transaction=NULL;
 		event.dialog=NULL;
 		event.response=msg;
@@ -317,7 +317,7 @@ static int channel_on_event(belle_sip_channel_listener_t *obj, belle_sip_channel
 static int channel_on_auth_requested(belle_sip_channel_listener_t *obj, belle_sip_channel_t *chan, const char* distinguished_name){
 	if (BELLE_SIP_IS_INSTANCE_OF(chan,belle_sip_tls_channel_t)) {
 		belle_sip_provider_t *prov=BELLE_SIP_PROVIDER(obj);
-		belle_sip_auth_event_t* auth_event = belle_sip_auth_event_create(NULL,NULL);
+		belle_sip_auth_event_t* auth_event = belle_sip_auth_event_create((belle_sip_object_t*)prov,NULL,NULL);
 		belle_sip_tls_channel_t *tls_chan=BELLE_SIP_TLS_CHANNEL(chan);
 		auth_event->mode=BELLE_SIP_AUTH_MODE_TLS;
 		belle_sip_auth_event_set_distinguished_name(auth_event,distinguished_name);
@@ -936,7 +936,7 @@ static void  belle_sip_provider_update_or_create_auth_context(belle_sip_provider
 
 int belle_sip_provider_add_authorization(belle_sip_provider_t *p, belle_sip_request_t* request,belle_sip_response_t *resp,belle_sip_list_t** auth_infos) {
 	belle_sip_header_call_id_t* call_id;
-	belle_sip_list_t* auth_context_lst;
+	belle_sip_list_t* auth_context_iterator;
 	belle_sip_list_t* authenticate_lst;
 	belle_sip_list_t* head;
 	belle_sip_header_www_authenticate_t* authenticate;
@@ -999,67 +999,64 @@ int belle_sip_provider_add_authorization(belle_sip_provider_t *p, belle_sip_requ
 	/*put authorization header if passwd found*/
 	call_id = belle_sip_message_get_header_by_type(BELLE_SIP_MESSAGE(request),belle_sip_header_call_id_t);
 	from = belle_sip_message_get_header_by_type(request,belle_sip_header_from_t);
-	if ((head=auth_context_lst = belle_sip_provider_get_auth_context_by_call_id(p,call_id))) {
-		/*we assume there no existing auth headers*/
-		for (;auth_context_lst!=NULL;auth_context_lst=auth_context_lst->next) {
-			/*clear auth info*/
-			auth_context=(authorization_context_t*)auth_context_lst->data;
-			auth_event = belle_sip_auth_event_create(auth_context->realm,from);
-			/*put data*/
-			/*call listener*/
-			BELLE_SIP_PROVIDER_INVOKE_LISTENERS(p->listeners,process_auth_requested,auth_event);
-			if (auth_event->passwd || auth_event->ha1) {
-				if (!auth_event->userid) {
-					/*if no userid, username = userid*/
+	head=belle_sip_provider_get_auth_context_by_call_id(p,call_id);
+	/*we assume there no existing auth headers*/
+	for (auth_context_iterator=head;auth_context_iterator!=NULL;auth_context_iterator=auth_context_iterator->next) {
+		/*clear auth info*/
+		auth_context=(authorization_context_t*)auth_context_iterator->data;
+		auth_event = belle_sip_auth_event_create((belle_sip_object_t*)p,auth_context->realm,from);
+		/*put data*/
+		/*call listener*/
+		BELLE_SIP_PROVIDER_INVOKE_LISTENERS(p->listeners,process_auth_requested,auth_event);
+		if (auth_event->passwd || auth_event->ha1) {
+			if (!auth_event->userid) {
+				/*if no userid, username = userid*/
 
-					belle_sip_auth_event_set_userid(auth_event,(const char*)auth_event->username);
-				}
-				belle_sip_message("Auth info found for [%s] realm [%s]",auth_event->userid,auth_event->realm);
-				if (auth_context->is_proxy) {
-					authorization=BELLE_SIP_HEADER_AUTHORIZATION(belle_sip_header_proxy_authorization_new());
-				} else {
-					authorization=belle_sip_header_authorization_new();
-				}
-				belle_sip_header_authorization_set_scheme(authorization,auth_context->scheme);
-				belle_sip_header_authorization_set_realm(authorization,auth_context->realm);
-				belle_sip_header_authorization_set_username(authorization,auth_event->userid);
-				belle_sip_header_authorization_set_nonce(authorization,auth_context->nonce);
-				belle_sip_header_authorization_set_qop(authorization,auth_context->qop);
-				belle_sip_header_authorization_set_opaque(authorization,auth_context->opaque);
-				belle_sip_header_authorization_set_uri(authorization,(belle_sip_uri_t*)belle_sip_request_get_uri(request));
-				if (auth_context->qop)
-					belle_sip_header_authorization_set_nonce_count(authorization,++auth_context->nonce_count);
-				if (auth_event->ha1) {
-					ha1=auth_event->ha1;
-				} else {
-					belle_sip_auth_helper_compute_ha1(auth_event->userid,auth_context->realm,auth_event->passwd, computed_ha1);
-					ha1=computed_ha1;
-				}
-				if (belle_sip_auth_helper_fill_authorization(authorization
-															,belle_sip_request_get_method(request)
-															,ha1)) {
-					belle_sip_object_unref(authorization);
-				} else
-					belle_sip_message_add_header(BELLE_SIP_MESSAGE(request),BELLE_SIP_HEADER(authorization));
-				result=1;
+				belle_sip_auth_event_set_userid(auth_event,(const char*)auth_event->username);
+			}
+			belle_sip_message("Auth info found for [%s] realm [%s]",auth_event->userid,auth_event->realm);
+			if (auth_context->is_proxy) {
+				authorization=BELLE_SIP_HEADER_AUTHORIZATION(belle_sip_header_proxy_authorization_new());
+			} else {
+				authorization=belle_sip_header_authorization_new();
+			}
+			belle_sip_header_authorization_set_scheme(authorization,auth_context->scheme);
+			belle_sip_header_authorization_set_realm(authorization,auth_context->realm);
+			belle_sip_header_authorization_set_username(authorization,auth_event->userid);
+			belle_sip_header_authorization_set_nonce(authorization,auth_context->nonce);
+			belle_sip_header_authorization_set_qop(authorization,auth_context->qop);
+			belle_sip_header_authorization_set_opaque(authorization,auth_context->opaque);
+			belle_sip_header_authorization_set_uri(authorization,(belle_sip_uri_t*)belle_sip_request_get_uri(request));
+			if (auth_context->qop)
+				belle_sip_header_authorization_set_nonce_count(authorization,++auth_context->nonce_count);
+			if (auth_event->ha1) {
+				ha1=auth_event->ha1;
+			} else {
+				belle_sip_auth_helper_compute_ha1(auth_event->userid,auth_context->realm,auth_event->passwd, computed_ha1);
+				ha1=computed_ha1;
+			}
+			if (belle_sip_auth_helper_fill_authorization(authorization
+														,belle_sip_request_get_method(request)
+														,ha1)) {
+				belle_sip_object_unref(authorization);
+			} else
+				belle_sip_message_add_header(BELLE_SIP_MESSAGE(request),BELLE_SIP_HEADER(authorization));
+			result=1;
 		} else {
 			belle_sip_message("No auth info found for call id [%s]",belle_sip_header_call_id_get_call_id(call_id));
 		}
-		/*provides auth info in any cases, uasefull even if found because auth info can contains wrong password*/
+		/*provides auth info in any cases, usefull even if found because auth info can contain wrong password*/
 		if (auth_infos) {
 			/*stored to give user information on realm/username which requires authentications*/
 			*auth_infos=belle_sip_list_append(*auth_infos,auth_event);
 		} else {
 			belle_sip_auth_event_destroy(auth_event);
 		}
-
-		}
-		belle_sip_list_free(head);
-	} else {
-		/*nothing to do*/
 	}
+	belle_sip_list_free(head);
 	return result;
 }
+
 void belle_sip_provider_set_recv_error(belle_sip_provider_t *prov, int recv_error) {
 	belle_sip_list_t *lps;
 	belle_sip_list_t *channels;
