@@ -4,7 +4,7 @@
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -130,7 +130,7 @@ belle_sip_response_t *belle_sip_transaction_get_response(const belle_sip_transac
 
 static void notify_timeout(belle_sip_transaction_t *t){
 	belle_sip_timeout_event_t ev;
-	ev.source=t->provider;
+	ev.source=(belle_sip_object_t*)t->provider;
 	ev.transaction=t;
 	ev.is_server_transaction=BELLE_SIP_OBJECT_IS_INSTANCE_OF(t,belle_sip_server_transaction_t);
 	BELLE_SIP_PROVIDER_INVOKE_LISTENERS_FOR_TRANSACTION(t,process_timeout,&ev);
@@ -245,7 +245,7 @@ void belle_sip_server_transaction_send_response(belle_sip_server_transaction_t *
 static void server_transaction_notify(belle_sip_server_transaction_t *t, belle_sip_request_t *req, belle_sip_dialog_t *dialog){
 	belle_sip_request_event_t event;
 
-	event.source=t->base.provider;
+	event.source=(belle_sip_object_t*)t->base.provider;
 	event.server_transaction=t;
 	event.dialog=dialog;
 	event.request=req;
@@ -441,7 +441,7 @@ void belle_sip_client_transaction_notify_response(belle_sip_client_transaction_t
 		return;
 	}
 
-	event.source=base->provider;
+	event.source=(belle_sip_object_t*)base->provider;
 	event.client_transaction=t;
 	event.dialog=dialog;
 	event.response=(belle_sip_response_t*)resp;
@@ -456,6 +456,10 @@ void belle_sip_client_transaction_add_response(belle_sip_client_transaction_t *t
 	BELLE_SIP_OBJECT_VPTR(t,belle_sip_client_transaction_t)->on_response(t,resp);
 }
 
+belle_sip_uri_t *belle_sip_client_transaction_get_route(belle_sip_client_transaction_t *t){
+	return t->preset_route;
+}
+
 static void client_transaction_destroy(belle_sip_client_transaction_t *t ){
 	if (t->preset_route) belle_sip_object_unref(t->preset_route);
 	if (t->next_hop) belle_sip_object_unref(t->next_hop);
