@@ -4,7 +4,7 @@
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -25,9 +25,8 @@
 #include <polarssl/ssl.h>
 #include <polarssl/version.h>
 #include <polarssl/error.h>
-
 #if POLARSSL_VERSION_NUMBER >= 0x01030000
-#include <polarssl/compat-1.2.h>
+#include <polarssl/x509.h>
 #endif
 #endif
 
@@ -35,14 +34,22 @@
 struct belle_sip_certificates_chain {
 	belle_sip_object_t objet;
 #ifdef HAVE_POLARSSL
+#if POLARSSL_VERSION_NUMBER < 0x01030000
 	x509_cert cert;
+#else
+	x509_crt cert;
+#endif
 #endif
 };
 
 struct belle_sip_signing_key {
 	belle_sip_object_t objet;
 #ifdef HAVE_POLARSSL
+#if POLARSSL_VERSION_NUMBER < 0x01030000
 	rsa_context key;
+#else
+	pk_context key;
+#endif
 #endif
 };
 
@@ -55,13 +62,18 @@ static int tls_process_data(belle_sip_channel_t *obj,unsigned int revents);
 struct belle_sip_tls_channel{
 	belle_sip_stream_channel_t base;
 	ssl_context sslctx;
+#if POLARSSL_VERSION_NUMBER < 0x01030000
 	x509_cert root_ca;
+#else
+	x509_crt root_ca;
+#endif
 	struct sockaddr_storage ss;
 	socklen_t socklen;
 	int socket_connected;
 	char *cur_debug_msg;
 	belle_sip_certificates_chain_t* client_cert_chain;
 	belle_sip_signing_key_t* client_cert_key;
+	belle_tls_verify_policy_t *verify_ctx;
 };
 
 static void tls_channel_close(belle_sip_tls_channel_t *obj){
@@ -78,16 +90,23 @@ static void tls_channel_uninit(belle_sip_tls_channel_t *obj){
 	if (sock!=(belle_sip_socket_t)-1)
 		tls_channel_close(obj);
 	ssl_free(&obj->sslctx);
+#if POLARSSL_VERSION_NUMBER < 0x01030000
 	x509_free(&obj->root_ca);
+#else
+	x509_crt_free(&obj->root_ca);
+#endif
 	if (obj->cur_debug_msg)
 		belle_sip_free(obj->cur_debug_msg);
+	belle_sip_object_unref(obj->verify_ctx);
 }
 
 static int tls_channel_send(belle_sip_channel_t *obj, const void *buf, size_t buflen){
 	belle_sip_tls_channel_t* channel = (belle_sip_tls_channel_t*)obj;
 	int err = ssl_write(&channel->sslctx,buf,buflen);
 	if (err<0){
-		belle_sip_error("Channel [%p]: error in ssl_write().",obj);
+		char tmp[256]={0};
+		error_strerror(err,tmp,sizeof(tmp));
+		belle_sip_error("Channel [%p]: ssl_write() error [%i]: %s",obj,err,tmp);
 	}
 	return err;
 }
@@ -95,8 +114,12 @@ static int tls_channel_send(belle_sip_channel_t *obj, const void *buf, size_t bu
 static int tls_channel_recv(belle_sip_channel_t *obj, void *buf, size_t buflen){
 	belle_sip_tls_channel_t* channel = (belle_sip_tls_channel_t*)obj;
 	int err = ssl_read(&channel->sslctx,buf,buflen);
+	if (err==POLARSSL_ERR_SSL_PEER_CLOSE_NOTIFY) return 0;
 	if (err<0){
-		belle_sip_error("Channel [%p]: error in ssl_read().",obj);
+		if (err==POLARSSL_ERR_NET_WANT_READ) return -BELLESIP_EWOULDBLOCK;
+		char tmp[256]={0};
+		error_strerror(err,tmp,sizeof(tmp));
+		belle_sip_error("Channel [%p]: ssl_read() error [%i]: %s",obj, err, tmp);
 	}
 	return err;
 }
@@ -153,14 +176,19 @@ static int tls_channel_handshake(belle_sip_tls_channel_t *channel) {
 				int err;
 #endif
 				char tmp[512]={0};
+#if POLARSSL_VERSION_NUMBER < 0x01030000
 				x509parse_cert_info(tmp,sizeof(tmp)-1,"",&channel->client_cert_chain->cert);
+#else
+				x509_crt_info(tmp,sizeof(tmp)-1,"",&channel->client_cert_chain->cert);
+#endif
 				belle_sip_message("Channel [%p]  found client  certificate:\n%s",channel,tmp);
 #if POLARSSL_VERSION_NUMBER < 0x01030000
 				ssl_set_own_cert(&channel->sslctx,&channel->client_cert_chain->cert,&channel->client_cert_key->key);
 #else
-				if ((err=ssl_set_own_cert_rsa(&channel->sslctx,&channel->client_cert_chain->cert,&channel->client_cert_key->key))) {
+                                /* allows public keys other than RSA */
+				if ((err=ssl_set_own_cert(&channel->sslctx,&channel->client_cert_chain->cert,&channel->client_cert_key->key))) {
 					error_strerror(err,tmp,sizeof(tmp)-1);
-					belle_sip_error("Channel [%p] cannot ssl_set_own_cert_rsa [%s]",channel,tmp);
+					belle_sip_error("Channel [%p] cannot ssl_set_own_cert [%s]",channel,tmp);
 				}
 
 				/*update own cert see ssl_handshake frompolarssl*/
@@ -172,33 +200,49 @@ static int tls_channel_handshake(belle_sip_tls_channel_t *channel) {
 	}
 	return ret;
 }
+
+static int tls_process_handshake(belle_sip_channel_t *obj){
+	belle_sip_tls_channel_t* channel=(belle_sip_tls_channel_t*)obj;
+	int err=tls_channel_handshake(channel);
+	if (err==0){
+		belle_sip_message("Channel [%p]: SSL handshake finished.",obj);
+		belle_sip_source_set_timeout((belle_sip_source_t*)obj,-1);
+		belle_sip_channel_set_ready(obj,(struct sockaddr*)&channel->ss,channel->socklen);
+	}else if (err==POLARSSL_ERR_NET_WANT_READ || err==POLARSSL_ERR_NET_WANT_WRITE){
+		belle_sip_message("Channel [%p]: SSL handshake in progress...",obj);
+	}else{
+		char tmp[128];
+		error_strerror(err,tmp,sizeof(tmp));
+		belle_sip_error("Channel [%p]: SSL handshake failed : %s",obj,tmp);
+		return -1;
+	}
+	return 0;
+}
+
 static int tls_process_data(belle_sip_channel_t *obj,unsigned int revents){
 	belle_sip_tls_channel_t* channel=(belle_sip_tls_channel_t*)obj;
-	int err;
 
 	if (obj->state == BELLE_SIP_CHANNEL_CONNECTING ) {
-		if (!channel->socket_connected && (revents & BELLE_SIP_EVENT_WRITE)) {
+		if (!channel->socket_connected) {
 			channel->socklen=sizeof(channel->ss);
-			if (finalize_stream_connection((belle_sip_stream_channel_t*)obj,(struct sockaddr*)&channel->ss,&channel->socklen)) {
+			if (finalize_stream_connection((belle_sip_stream_channel_t*)obj,revents,(struct sockaddr*)&channel->ss,&channel->socklen)) {
 				goto process_error;
 			}
-			belle_sip_source_set_events((belle_sip_source_t*)channel,BELLE_SIP_EVENT_READ|BELLE_SIP_EVENT_ERROR);
-			channel->socket_connected=1;
 			belle_sip_message("Channel [%p]: Connected at TCP level, now doing TLS handshake",obj);
-		}
-		err=tls_channel_handshake(channel) /*ssl_handshake(&channel->sslctx)*/;
-		if (err==0){
-			belle_sip_message("Channel [%p]: SSL handshake finished.",obj);
-			belle_sip_channel_set_ready(obj,(struct sockaddr*)&channel->ss,channel->socklen);
-		}else if (err==POLARSSL_ERR_NET_WANT_READ || err==POLARSSL_ERR_NET_WANT_WRITE){
-			belle_sip_message("Channel [%p]: SSL handshake in progress...",obj);
+			channel->socket_connected=1;
+			belle_sip_source_set_events((belle_sip_source_t*)channel,BELLE_SIP_EVENT_READ|BELLE_SIP_EVENT_ERROR);
+			belle_sip_source_set_timeout((belle_sip_source_t*)obj,belle_sip_stack_get_transport_timeout(obj->stack));
+			if (tls_process_handshake(obj)==-1) goto process_error;
 		}else{
-			char tmp[128];
-			error_strerror(err,tmp,sizeof(tmp));
-			belle_sip_error("Channel [%p]: SSL handshake failed : %s",obj,tmp);
-			goto process_error;
+			if (revents & BELLE_SIP_EVENT_READ){
+				if (tls_process_handshake(obj)==-1) goto process_error;
+			}else if (revents==BELLE_SIP_EVENT_TIMEOUT){
+				belle_sip_error("channel [%p]: SSL handshake took too much time.",obj);
+				goto process_error;
+			}else{
+				belle_sip_warning("channeEHHCXCCCl [%p]: unexpected event [%i] during TLS handshake.",obj,revents);
+			}
 		}
-		
 	} else if ( obj->state == BELLE_SIP_CHANNEL_READY) {
 		return belle_sip_channel_process_data(obj,revents);
 	} else {
@@ -269,17 +313,25 @@ static const char *polarssl_certflags_to_string(char *buf, size_t size, int flag
 	return buf;
 }
 
+#if POLARSSL_VERSION_NUMBER < 0x01030000
 static int belle_sip_ssl_verify(void *data , x509_cert *cert , int depth, int *flags){
-	belle_sip_tls_listening_point_t *lp=(belle_sip_tls_listening_point_t*)data;
+#else
+static int belle_sip_ssl_verify(void *data , x509_crt *cert , int depth, int *flags){
+#endif
+	belle_tls_verify_policy_t *verify_ctx=(belle_tls_verify_policy_t*)data;
 	char tmp[512];
 	char flags_str[128];
 	
+#if POLARSSL_VERSION_NUMBER < 0x01030000
 	x509parse_cert_info(tmp,sizeof(tmp),"",cert);
+#else
+	x509_crt_info(tmp,sizeof(tmp),"",cert);
+#endif
 	belle_sip_message("Found certificate depth=[%i], flags=[%s]:\n%s",
 		depth,polarssl_certflags_to_string(flags_str,sizeof(flags_str),*flags),tmp);
-	if (lp->verify_exceptions==BELLE_SIP_TLS_LISTENING_POINT_BADCERT_ANY_REASON){
+	if (verify_ctx->exception_flags==BELLE_TLS_VERIFY_ANY_REASON){
 		*flags=0;
-	}else if (lp->verify_exceptions & BELLE_SIP_TLS_LISTENING_POINT_BADCERT_CN_MISMATCH){
+	}else if (verify_ctx->exception_flags & BELLE_TLS_VERIFY_CN_MISMATCH){
 		*flags&=~BADCERT_CN_MISMATCH;
 	}
 	return 0;
@@ -289,12 +341,20 @@ static int belle_sip_tls_channel_load_root_ca(belle_sip_tls_channel_t *obj, cons
 	struct stat statbuf; 
 	if (stat(path,&statbuf)==0){
 		if (statbuf.st_mode & S_IFDIR){
+#if POLARSSL_VERSION_NUMBER < 0x01030000
 			if (x509parse_crtpath(&obj->root_ca,path)<0){
+#else
+			if (x509_crt_parse_path(&obj->root_ca,path)<0){
+#endif
 				belle_sip_error("Failed to load root ca from directory %s",path);
 				return -1;
 			}
 		}else{
+#if POLARSSL_VERSION_NUMBER < 0x01030000
 			if (x509parse_crtfile(&obj->root_ca,path)<0){
+#else
+			if (x509_crt_parse_file(&obj->root_ca,path)<0){
+#endif
 				belle_sip_error("Failed to load root ca from file %s",path);
 				return -1;
 			}
@@ -335,12 +395,12 @@ static void ssl_debug_to_belle_sip(void *context, int level, const char *str){
 
 #endif
 
-belle_sip_channel_t * belle_sip_channel_new_tls(belle_sip_tls_listening_point_t *lp,const char *bindip, int localport, const char *peer_cname, const char *dest, int port){
+belle_sip_channel_t * belle_sip_channel_new_tls(belle_sip_stack_t *stack, belle_tls_verify_policy_t *verify_ctx,const char *bindip, int localport, const char *peer_cname, const char *dest, int port){
 	belle_sip_tls_channel_t *obj=belle_sip_object_new(belle_sip_tls_channel_t);
 	belle_sip_stream_channel_t* super=(belle_sip_stream_channel_t*)obj;
 
 	belle_sip_stream_channel_init_client(super
-					,((belle_sip_listening_point_t*)lp)->stack
+					,stack
 					,bindip,localport,peer_cname,dest,port);
 	ssl_init(&obj->sslctx);
 #ifdef ENABLE_POLARSSL_LOGS
@@ -349,11 +409,12 @@ belle_sip_channel_t * belle_sip_channel_new_tls(belle_sip_tls_listening_point_t
 	ssl_set_endpoint(&obj->sslctx,SSL_IS_CLIENT);
 	ssl_set_authmode(&obj->sslctx,SSL_VERIFY_REQUIRED);
 	ssl_set_bio(&obj->sslctx,polarssl_read,obj,polarssl_write,obj);
-	if (lp->root_ca && belle_sip_tls_channel_load_root_ca(obj,lp->root_ca)==0){
+	if (verify_ctx->root_ca && belle_sip_tls_channel_load_root_ca(obj,verify_ctx->root_ca)==0){
 		ssl_set_ca_chain(&obj->sslctx,&obj->root_ca,NULL,super->base.peer_cname ? super->base.peer_cname : super->base.peer_name );
 	}
 	ssl_set_rng(&obj->sslctx,random_generator,NULL);
-	ssl_set_verify(&obj->sslctx,belle_sip_ssl_verify,lp);
+	ssl_set_verify(&obj->sslctx,belle_sip_ssl_verify,verify_ctx);
+	obj->verify_ctx=(belle_tls_verify_policy_t*)belle_sip_object_ref(verify_ctx);
 	return (belle_sip_channel_t*)obj;
 }
 
@@ -389,7 +450,11 @@ static int belle_sip_certificate_fill(belle_sip_certificates_chain_t* certificat
 #ifdef HAVE_POLARSSL
 
 	int err;
+#if POLARSSL_VERSION_NUMBER < 0x01030000
 	if ((err=x509parse_crt(&certificate->cert,(const unsigned char *)buff,size)) <0) {
+#else
+	if ((err=x509_crt_parse(&certificate->cert,(const unsigned char *)buff,size)) <0) {
+#endif
 		char tmp[128];
 		error_strerror(err,tmp,sizeof(tmp));
 		belle_sip_error("cannot parse x509 cert because [%s]",tmp);
@@ -405,7 +470,11 @@ static int belle_sip_certificate_fill_from_file(belle_sip_certificates_chain_t*
 #ifdef HAVE_POLARSSL
 
 	int err;
+#if POLARSSL_VERSION_NUMBER < 0x01030000
 	if ((err=x509parse_crtfile(&certificate->cert, path)) <0) {
+#else
+	if ((err=x509_crt_parse_file(&certificate->cert, path)) <0) {
+#endif
 		char tmp[128];
 		error_strerror(err,tmp,sizeof(tmp));
 		belle_sip_error("cannot parse x509 cert because [%s]",tmp);
@@ -443,7 +512,11 @@ belle_sip_certificates_chain_t* belle_sip_certificates_chain_parse_file(const ch
 
 static void belle_sip_certificates_chain_destroy(belle_sip_certificates_chain_t *certificate){
 #ifdef HAVE_POLARSSL
+#if POLARSSL_VERSION_NUMBER < 0x01030000
 	x509_free(&certificate->cert);
+#else
+	x509_crt_free(&certificate->cert);
+#endif
 #endif
 }
 
@@ -461,10 +534,23 @@ belle_sip_signing_key_t* belle_sip_signing_key_parse(const char* buff, size_t si
 #ifdef HAVE_POLARSSL
 	belle_sip_signing_key_t* signing_key = belle_sip_object_new(belle_sip_signing_key_t);
 	int err;
+#if POLARSSL_VERSION_NUMBER < 0x01030000
 	if ((err=x509parse_key(&signing_key->key,(const unsigned char *)buff,size,(const unsigned char*)passwd,passwd?strlen(passwd):0)) <0) {
+#else
+    pk_init(&signing_key->key);
+    /* for API v1.3 or greater also parses public keys other than RSA */
+	err=pk_parse_key(&signing_key->key,(const unsigned char *)buff,size,(const unsigned char*)passwd,passwd?strlen(passwd):0);
+     /* make sure cipher is RSA to be consistent with API v1.2 */
+    if(err==0 && !pk_can_do(&signing_key->key,POLARSSL_PK_RSA))
+    	err=POLARSSL_ERR_PK_TYPE_MISMATCH;
+	if (err<0) {
+#endif
 		char tmp[128];
 		error_strerror(err,tmp,sizeof(tmp));
-		belle_sip_error("cannot parse rsa key because [%s]",tmp);
+		belle_sip_error("cannot parse public key because [%s]",tmp);
+#if POLARSSL_VERSION_NUMBER >= 0x01030000
+                pk_free(&signing_key->key);
+#endif
 		belle_sip_object_unref(signing_key);
 		return NULL;
 	}
@@ -478,13 +564,27 @@ belle_sip_signing_key_t* belle_sip_signing_key_parse_file(const char* path,const
 #ifdef HAVE_POLARSSL
 	belle_sip_signing_key_t* signing_key = belle_sip_object_new(belle_sip_signing_key_t);
 	int err;
-	if ((err=x509parse_keyfile(&signing_key->key,(const unsigned char *)path, passwd)) <0) {
+#if POLARSSL_VERSION_NUMBER < 0x01030000
+	if ((err=x509parse_keyfile(&signing_key->key,path, passwd)) <0) {
+#else
+    pk_init(&signing_key->key);
+    /* for API v1.3 or greater also parses public keys other than RSA */
+    err=pk_parse_keyfile(&signing_key->key,path, passwd);
+    /* make sure cipher is RSA to be consistent with API v1.2 */
+    if(err==0 && !pk_can_do(&signing_key->key,POLARSSL_PK_RSA))
+    	err=POLARSSL_ERR_PK_TYPE_MISMATCH;
+	if (err<0) {
+#endif
 		char tmp[128];
 		error_strerror(err,tmp,sizeof(tmp));
-		belle_sip_error("cannot parse rsa key because [%s]",tmp);
+		belle_sip_error("cannot parse public key because [%s]",tmp);
+#if POLARSSL_VERSION_NUMBER >= 0x01030000
+        pk_free(&signing_key->key);
+#endif
 		belle_sip_object_unref(signing_key);
 		return NULL;
 	}
+
 	return signing_key;
 #else /*HAVE_POLARSSL*/
 	return NULL;
@@ -494,7 +594,11 @@ belle_sip_signing_key_t* belle_sip_signing_key_parse_file(const char* path,const
 
 static void belle_sip_signing_key_destroy(belle_sip_signing_key_t *signing_key){
 #ifdef HAVE_POLARSSL
+#if POLARSSL_VERSION_NUMBER < 0x01030000
 	rsa_free(&signing_key->key);
+#else
+	pk_free(&signing_key->key);
+#endif
 #endif
 }
 
