@@ -4,7 +4,7 @@
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -28,11 +28,16 @@
 const char *test_domain="test.linphone.org";
 const char *auth_domain="sip.linphone.org";
 const char *client_auth_domain="client.example.org";
-const char *client_auth_outband_proxy="sips:sip2.linphone.org:5063";
+const char *client_auth_outbound_proxy="sips:sip2.linphone.org:5063";
+const char *no_server_running_here="sip:test.linphone.org:3;transport=tcp";
+const char *no_response_here="sip:78.220.48.77:3;transport=tcp";
+const char *test_domain_tls_to_tcp="sip:test.linphone.org:5060;transport=tls";
 
 static int is_register_ok;
 static int number_of_challenge;
 static int using_transaction;
+static int io_error_count=0;
+
 belle_sip_stack_t * stack;
 belle_sip_provider_t *prov;
 static belle_sip_listener_t* l;
@@ -50,8 +55,9 @@ static void process_dialog_terminated(void *user_ctx, const belle_sip_dialog_ter
 static void process_io_error(void *user_ctx, const belle_sip_io_error_event_t *event){
 	BELLESIP_UNUSED(user_ctx);
 	BELLESIP_UNUSED(event);
-	belle_sip_message("process_io_error");
+	belle_sip_message("process_io_error, exiting main loop");
 	belle_sip_main_loop_quit(belle_sip_stack_get_main_loop(stack));
+	io_error_count++;
 	/*CU_ASSERT(CU_FALSE);*/
 }
 
@@ -72,14 +78,17 @@ static void process_response_event(void *user_ctx, const belle_sip_response_even
 	if (status==401) {
 		belle_sip_header_cseq_t* cseq;
 		belle_sip_client_transaction_t *t;
+		belle_sip_uri_t *dest;
+		
 		CU_ASSERT_NOT_EQUAL_FATAL(number_of_challenge,2);
 		CU_ASSERT_PTR_NOT_NULL_FATAL(belle_sip_response_event_get_client_transaction(event)); /*require transaction mode*/
+		dest=belle_sip_client_transaction_get_route(belle_sip_response_event_get_client_transaction(event));
 		request=belle_sip_transaction_get_request(BELLE_SIP_TRANSACTION(belle_sip_response_event_get_client_transaction(event)));
 		cseq=(belle_sip_header_cseq_t*)belle_sip_message_get_header(BELLE_SIP_MESSAGE(request),BELLE_SIP_CSEQ);
 		belle_sip_header_cseq_set_seq_number(cseq,belle_sip_header_cseq_get_seq_number(cseq)+1);
 		CU_ASSERT_TRUE_FATAL(belle_sip_provider_add_authorization(prov,request,belle_sip_response_event_get_response(event),NULL));
 		t=belle_sip_provider_create_client_transaction(prov,request);
-		belle_sip_client_transaction_send_request(t);
+		belle_sip_client_transaction_send_request_to(t,dest);
 		number_of_challenge++;
 		authorized_request=request;
 		belle_sip_object_ref(authorized_request);
@@ -103,7 +112,7 @@ static void process_transaction_terminated(void *user_ctx, const belle_sip_trans
 	belle_sip_message("process_transaction_terminated");
 }
 
-static const char* client_cert = /*for URI:sip:tester@client.example.org*/
+const char* belle_sip_tester_client_cert = /*for URI:sip:tester@client.example.org*/
 		"-----BEGIN CERTIFICATE-----\n"
 		"MIIDYzCCAsygAwIBAgIBCDANBgkqhkiG9w0BAQUFADCBuzELMAkGA1UEBhMCRlIx\n"
 		"EzARBgNVBAgMClNvbWUtU3RhdGUxETAPBgNVBAcMCEdyZW5vYmxlMSIwIAYDVQQK\n"
@@ -127,7 +136,7 @@ static const char* client_cert = /*for URI:sip:tester@client.example.org*/
 		"-----END CERTIFICATE-----";
 
 
-static const char* private_key =
+const char* belle_sip_tester_private_key =
 		"-----BEGIN ENCRYPTED PRIVATE KEY-----\n"
 		"MIICxjBABgkqhkiG9w0BBQ0wMzAbBgkqhkiG9w0BBQwwDgQIbEHnQwhgRwoCAggA\n"
 		"MBQGCCqGSIb3DQMHBAgmrtCEBCP9kASCAoCq9EKInROalaBSLWY44U4RVAC+CKdx\n"
@@ -146,23 +155,26 @@ static const char* private_key =
 		"WEuSefVM4xgB+mfQauAJu2N9yKhzXOytslZflpa06qJedlLYFk9njvcv\n"
 		"-----END ENCRYPTED PRIVATE KEY-----\n";
 
-static const char* private_key_passwd="secret";
+const char* belle_sip_tester_private_key_passwd="secret";
 
 
 static void process_auth_requested(void *user_ctx, belle_sip_auth_event_t *event){
 	BELLESIP_UNUSED(user_ctx);
 	if (belle_sip_auth_event_get_mode(event) == BELLE_SIP_AUTH_MODE_HTTP_DIGEST) {
+		const char *username = belle_sip_auth_event_get_username(event);
+		const char *realm = belle_sip_auth_event_get_realm(event);
 		belle_sip_message("process_auth_requested requested for [%s@%s]"
-				,belle_sip_auth_event_get_username(event)
-				,belle_sip_auth_event_get_realm(event));
+				,username?username:""
+				,realm?realm:"");
 		belle_sip_auth_event_set_passwd(event,"secret");
 	} else if (belle_sip_auth_event_get_mode(event) == BELLE_SIP_AUTH_MODE_TLS) {
-		belle_sip_certificates_chain_t* cert = belle_sip_certificates_chain_parse(client_cert,strlen(client_cert),BELLE_SIP_CERTIFICATE_RAW_FORMAT_PEM);
-		belle_sip_signing_key_t* key = belle_sip_signing_key_parse(private_key,strlen(private_key),private_key_passwd);
+		const char *distinguished_name = NULL;
+		belle_sip_certificates_chain_t* cert = belle_sip_certificates_chain_parse(belle_sip_tester_client_cert,strlen(belle_sip_tester_client_cert),BELLE_SIP_CERTIFICATE_RAW_FORMAT_PEM);
+		belle_sip_signing_key_t* key = belle_sip_signing_key_parse(belle_sip_tester_private_key,strlen(belle_sip_tester_private_key),belle_sip_tester_private_key_passwd);
 		belle_sip_auth_event_set_client_certificates_chain(event,cert);
 		belle_sip_auth_event_set_signing_key(event,key);
-		belle_sip_message("process_auth_requested requested for  DN[%s]"
-							,belle_sip_auth_event_get_distinguished_name(event));
+		distinguished_name = belle_sip_auth_event_get_distinguished_name(event);
+		belle_sip_message("process_auth_requested requested for  DN[%s]",distinguished_name?distinguished_name:"");
 
 	} else {
 		belle_sip_error("Unexpected auth mode");
@@ -275,6 +287,7 @@ belle_sip_request_t* try_register_user_at_domain(belle_sip_stack_t * stack
 	                    belle_sip_header_via_new(),
 	                    70);
 	is_register_ok=0;
+	io_error_count=0;
 	using_transaction=0;
 	belle_sip_message_add_header(BELLE_SIP_MESSAGE(req),BELLE_SIP_HEADER(belle_sip_header_expires_create(600)));
 	belle_sip_message_add_header(BELLE_SIP_MESSAGE(req),BELLE_SIP_HEADER(belle_sip_header_contact_new()));
@@ -284,7 +297,7 @@ belle_sip_request_t* try_register_user_at_domain(belle_sip_stack_t * stack
 		belle_sip_client_transaction_t *t=belle_sip_provider_create_client_transaction(prov,req);
 		belle_sip_client_transaction_send_request_to(t,outbound?belle_sip_uri_parse(outbound):NULL);
 	}else belle_sip_provider_send_request(prov,req);
-	for(i=0;!is_register_ok && i<2 ;i++)
+	for(i=0;!is_register_ok && i<2 && io_error_count==0;i++)
 		belle_sip_stack_sleep(stack,5000);
 	CU_ASSERT_EQUAL(is_register_ok,success_expected);
 	if (success_expected) CU_ASSERT_EQUAL(using_transaction,use_transaction);
@@ -300,8 +313,8 @@ belle_sip_request_t* register_user_at_domain(belle_sip_stack_t * stack
 					,int use_transaction
 					,const char* username
 					,const char* domain
-					,const char* outband) {
-	return try_register_user_at_domain(stack,prov,transport,use_transaction,username,domain,outband,1);
+					,const char* outbound) {
+	return try_register_user_at_domain(stack,prov,transport,use_transaction,username,domain,outbound,1);
 
 }
 
@@ -397,6 +410,7 @@ static void test_bad_request(void) {
 	belle_sip_header_route_t* route;
 	belle_sip_header_to_t* to = belle_sip_header_to_create2("sip:toto@titi.com",NULL);
 	belle_sip_listener_callbacks_t cbs;
+	belle_sip_listening_point_t *lp=belle_sip_provider_get_listening_point(prov,"TCP");
 	int bad_request_response_received=0;
 	memset(&cbs,0,sizeof(cbs));
 
@@ -429,6 +443,8 @@ static void test_bad_request(void) {
 	CU_ASSERT_TRUE(bad_request_response_received==1);
 	belle_sip_provider_remove_sip_listener(prov,bad_req_listener);
 	belle_sip_object_unref(bad_req_listener);
+	
+	belle_sip_listening_point_clean_channels(lp);
 }
 
 static void test_register_authenticate(void) {
@@ -464,7 +480,7 @@ static void test_register_client_authenticated(void) {
 	/*we don't care to check sercer cert*/
 	belle_sip_tls_listening_point_set_verify_exceptions(	(belle_sip_tls_listening_point_t*)belle_sip_provider_get_listening_point(prov,"tls")
 															,BELLE_SIP_TLS_LISTENING_POINT_BADCERT_ANY_REASON);
-	reg=register_user_at_domain(stack, prov, "tls",1,"tester",client_auth_domain,client_auth_outband_proxy);
+	reg=register_user_at_domain(stack, prov, "tls",1,"tester",client_auth_domain,client_auth_outbound_proxy);
 	if (authorized_request) {
 		unregister_user(stack,prov,authorized_request,1);
 		belle_sip_object_unref(authorized_request);
@@ -472,6 +488,54 @@ static void test_register_client_authenticated(void) {
 	if (reg) belle_sip_object_unref(reg);
 }
 
+static void test_connection_failure(void){
+	belle_sip_request_t *req;
+	io_error_count=0;
+	req=try_register_user_at_domain(stack, prov, "TCP",1,"tester","sip.linphone.org",no_server_running_here,0);
+	CU_ASSERT_TRUE(io_error_count>=1);
+	if (req) belle_sip_object_unref(req);
+}
+
+static void test_connection_too_long(void){
+	belle_sip_request_t *req;
+	io_error_count=0;
+	int orig=belle_sip_stack_get_transport_timeout(stack);
+	belle_sip_stack_set_transport_timeout(stack,2000);
+	req=try_register_user_at_domain(stack, prov, "TCP",1,"tester","sip.linphone.org",no_response_here,0);
+	CU_ASSERT_TRUE(io_error_count>=1);
+	belle_sip_stack_set_transport_timeout(stack,orig);
+	if (req) belle_sip_object_unref(req);
+}
+
+static void test_tls_to_tcp(void){
+	io_error_count=0;
+	belle_sip_request_t *req;
+	int orig=belle_sip_stack_get_transport_timeout(stack);
+	belle_sip_stack_set_transport_timeout(stack,2000);
+	req=try_register_user_at_domain(stack, prov, "TLS",1,"tester",test_domain,test_domain_tls_to_tcp,0);
+	if (req){
+		CU_ASSERT_TRUE(io_error_count>=1);
+		belle_sip_object_unref(req);
+	}
+	belle_sip_stack_set_transport_timeout(stack,orig);
+}
+
+static void register_dns_srv_tcp(void){
+	belle_sip_request_t *req;
+	io_error_count=0;
+	req=try_register_user_at_domain(stack, prov, "TCP",1,"tester",client_auth_domain,"sip:linphone.net;transport=tcp",1);
+	CU_ASSERT_TRUE(io_error_count==0);
+	if (req) belle_sip_object_unref(req);
+}
+
+static void register_dns_srv_tls(void){
+	belle_sip_request_t *req;
+	io_error_count=0;
+	req=try_register_user_at_domain(stack, prov, "TLS",1,"tester",client_auth_domain,"sip:linphone.net;transport=tls",1);
+	CU_ASSERT_TRUE(io_error_count==0);
+	if (req) belle_sip_object_unref(req);
+}
+
 
 test_t register_tests[] = {
 	{ "Stateful UDP", stateful_register_udp },
@@ -487,7 +551,12 @@ test_t register_tests[] = {
 	{ "Bad TCP request", test_bad_request },
 	{ "Authenticate", test_register_authenticate },
 	{ "TLS client cert authentication", test_register_client_authenticated },
-	{ "Channel inactive", test_register_channel_inactive }
+	{ "Channel inactive", test_register_channel_inactive },
+	{ "TCP connection failure", test_connection_failure },
+	{ "TCP connection too long", test_connection_too_long },
+	{ "TLS connection to TCP server", test_tls_to_tcp },
+	{ "Register with DNS SRV failover TCP", register_dns_srv_tcp },
+	{ "Register with DNS SRV failover TLS", register_dns_srv_tls }
 };
 
 test_suite_t register_test_suite = {
