@@ -4,7 +4,7 @@
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
+    the Free Software Foundation, either version 2 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -38,6 +38,11 @@
 #define SRV_DOMAIN		"linphone.org"
 #define SIP_PORT		5060
 
+#if __QNX__
+	#define WRITE_FILE_PATH	"./tmp/"
+#else
+	#define WRITE_FILE_PATH
+#endif
 
 typedef struct endpoint {
 	belle_sip_stack_t* stack;
@@ -243,7 +248,14 @@ static void ipv4_a_query_multiple_results(void) {
 
 static void ipv4_a_query_with_v4mapped_results(void) {
 	int timeout;
-	endpoint_t *client = create_endpoint();
+	endpoint_t *client;
+
+	if (!belle_sip_tester_ipv6_available()){
+		belle_sip_warning("Test skipped, IPv6 connectivity not available.");
+		return;
+	}
+
+	client = create_endpoint();
 
 	CU_ASSERT_PTR_NOT_NULL_FATAL(client);
 	timeout = belle_sip_stack_get_dns_timeout(client->stack);
@@ -260,8 +272,14 @@ static void ipv4_a_query_with_v4mapped_results(void) {
 static void ipv6_aaaa_query(void) {
 	struct addrinfo *ai;
 	int timeout;
-	endpoint_t *client = create_endpoint();
+	endpoint_t *client;
 
+	if (!belle_sip_tester_ipv6_available()){
+		belle_sip_warning("Test skipped, IPv6 connectivity not available.");
+		return;
+	}
+
+	client = create_endpoint();
 	CU_ASSERT_PTR_NOT_NULL_FATAL(client);
 	timeout = belle_sip_stack_get_dns_timeout(client->stack);
 	client->resolver_ctx = belle_sip_stack_resolve_a(client->stack, IPV6_SIP_DOMAIN, SIP_PORT, AF_INET6, a_resolve_done, client);
@@ -408,7 +426,7 @@ static void no_query_needed(void) {
 }
 
 static void set_custom_resolv_conf(belle_sip_stack_t *stack, const char *ns[3]){
-	FILE *f=fopen("tmp_resolv.conf","w");
+	FILE *f=fopen(WRITE_FILE_PATH "tmp_resolv.conf","w");
 	CU_ASSERT_PTR_NOT_NULL(f);
 	if (f){
 		int i;
@@ -419,7 +437,7 @@ static void set_custom_resolv_conf(belle_sip_stack_t *stack, const char *ns[3]){
 		}
 		fclose(f);
 	}
-	belle_sip_stack_set_dns_resolv_conf_file(stack,"tmp_resolv.conf");
+	belle_sip_stack_set_dns_resolv_conf_file(stack,WRITE_FILE_PATH "tmp_resolv.conf");
 }
 
 static void dns_fallback(void) {
