@@ -22,20 +22,20 @@
 
 #include <stdio.h>
 #include "CUnit/Basic.h"
-#if HAVE_CU_CURSES
+#ifdef HAVE_CU_CURSES
 #include "CUnit/CUCurses.h"
 #endif
 #include <belle-sip/belle-sip.h>
 
 extern const char *test_domain;
 extern const char *auth_domain;
-
+const char *root_ca_path = NULL;
 
 static test_suite_t **test_suite = NULL;
 static int nb_test_suites = 0;
 
 
-#if HAVE_CU_CURSES
+#ifdef HAVE_CU_CURSES
 	static unsigned char curses = 0;
 #endif
 
@@ -127,7 +127,8 @@ void belle_sip_tester_init(void) {
 	belle_sip_object_enable_marshal_check(TRUE);
 	ipv6_available=_belle_sip_tester_ipv6_available();
 	add_test_suite(&cast_test_suite);
-	add_test_suite(&uri_test_suite);
+	add_test_suite(&sip_uri_test_suite);
+	add_test_suite(&generic_uri_test_suite);
 	add_test_suite(&headers_test_suite);
 	add_test_suite(&core_test_suite);
 	add_test_suite(&sdp_test_suite);
@@ -137,6 +138,7 @@ void belle_sip_tester_init(void) {
 	add_test_suite(&register_test_suite);
 	add_test_suite(&dialog_test_suite);
 	add_test_suite(&refresher_test_suite);
+	add_test_suite(&http_test_suite);
 }
 
 
@@ -149,7 +151,7 @@ void belle_sip_tester_uninit(void) {
 	}
 }
 int belle_sip_tester_run_tests(const char *suite_name, const char *test_name) {
-	int i;
+	int i,ret;
 	belle_sip_object_pool_t *pool;
 	
 	/* initialize the CUnit test registry */
@@ -161,20 +163,28 @@ int belle_sip_tester_run_tests(const char *suite_name, const char *test_name) {
 	}
 	pool=belle_sip_object_pool_push();
 	
-#if HAVE_CU_GET_SUITE
+#ifdef HAVE_CU_GET_SUITE
 	if (suite_name){
 		CU_pSuite suite;
 		CU_basic_set_mode(CU_BRM_VERBOSE);
 		suite=CU_get_suite(suite_name);
+		if (suite==NULL){
+			fprintf(stderr,"There is no suite named '%s'",suite_name);
+			exit(-1);
+		}
 		if (test_name) {
 			CU_pTest test=CU_get_test_by_name(test_name, suite);
+			if (test==NULL){
+				fprintf(stderr,"There is no test named '%s'",suite_name);
+				exit(-1);
+			}
 			CU_basic_run_test(suite, test);
 		} else
 			CU_basic_run_suite(suite);
 	} else
 #endif
 	{
-#if HAVE_CU_CURSES
+#ifdef HAVE_CU_CURSES
 		if (curses) {
 			/* Run tests using the CUnit curses interface */
 			CU_curses_run_tests();
@@ -189,67 +199,101 @@ int belle_sip_tester_run_tests(const char *suite_name, const char *test_name) {
 	}
 
 	belle_sip_object_unref(pool);
-	
+	ret=CU_get_number_of_tests_failed()!=0;
 	CU_cleanup_registry();
-	return CU_get_error();
+	return ret;
 }
 
 
+void helper(const char *name) {
+	fprintf(stderr,"%s \t--help\n"
+		"\t\t\t--verbose\n"
+		"\t\t\t--domain <test sip domain>\n"
+		"\t\t\t--auth-domain <test auth domain>\n"
+		"\t\t\t--root-ca <root ca file path>\n"
+#ifdef HAVE_CU_GET_SUITE
+		"\t\t\t--list-suites\n"
+		"\t\t\t--list-tests <suite>\n"
+		"\t\t\t--suite <suite name>\n"
+		"\t\t\t--test <test name>\n"
+#endif
+#ifdef HAVE_CU_CURSES
+		"\t\t\t--curses\n"
+#endif
+		, name);
+}
+
+#define CHECK_ARG(argument, index, argc) \
+	if (index >= argc) { \
+		fprintf(stderr, "Missing argument for \"%s\"\n", argument); \
+		return -1; \
+	}
+
 #ifndef WINAPI_FAMILY_PHONE_APP
 int main (int argc, char *argv[]) {
 	int i;
 	int ret;
-	char *suite_name=NULL;
-	char *test_name=NULL;
+	const char *suite_name=NULL;
+	const char *test_name=NULL;
 	const char *env_domain=getenv("TEST_DOMAIN");
 
+	belle_sip_tester_init();
 
 	if (env_domain)
 		test_domain=env_domain;
 
 	for(i=1;i<argc;++i){
 		if (strcmp(argv[i],"--help")==0){
-				fprintf(stderr,"%s \t--help\n"
-						"\t\t\t--verbose\n"
-						"\t\t\t--domain <test sip domain>\n"
-						"\t\t\t--auth-domain <test auth domain>\n"
-#if HAVE_CU_GET_SUITE
-						"\t\t\t--suite <suite name>\n"
-						"\t\t\t--test <test name>\n"
-#endif
-#if HAVE_CU_CURSES
-						"\t\t\t--curses\n"
-#endif
-						, argv[0]);
-				return 0;
+			helper(argv[0]);
+			return 0;
 		}else if (strcmp(argv[i],"--verbose")==0){
 			belle_sip_set_log_level(BELLE_SIP_LOG_DEBUG);
 		}else if (strcmp(argv[i],"--domain")==0){
-			i++;
+			CHECK_ARG("--domain", ++i, argc);
 			test_domain=argv[i];
+		}else if (strcmp(argv[i],"--auth-domain")==0){
+			CHECK_ARG("--auth-domain", ++i, argc);
+			auth_domain=argv[i];
+		} else if (strcmp(argv[i], "--root-ca") == 0) {
+			CHECK_ARG("--root-ca", ++i, argc);
+			root_ca_path = argv[i];
 		}
-		else if (strcmp(argv[i],"--auth-domain")==0){
-					i++;
-					auth_domain=argv[i];
-		}
-#if HAVE_CU_GET_SUITE
-		else if (strcmp(argv[i],"--test")==0){
-			i++;
+#ifdef HAVE_CU_GET_SUITE
+		else if (strcmp(argv[i],"--list-suites")==0){
+			int j;
+			for(j = 0; j < belle_sip_tester_nb_test_suites(); j++) {
+				suite_name = belle_sip_tester_test_suite_name(j);
+				fprintf(stdout, "%s\n", suite_name);
+			}
+			return 0;
+		} else if (strcmp(argv[i],"--list-tests")==0){
+			int j;
+			CHECK_ARG("--list-tests", ++i, argc);
+			suite_name = argv[i];
+			for(j = 0; j < belle_sip_tester_nb_tests(suite_name);j++) {
+				test_name = belle_sip_tester_test_name(suite_name, j);
+				fprintf(stdout, "%s\n", test_name);
+			}
+			return 0;
+		} else if (strcmp(argv[i],"--test")==0){
+			CHECK_ARG("--test", ++i, argc);
 			test_name=argv[i];
 		}else if (strcmp(argv[i],"--suite")==0){
-			i++;
+			CHECK_ARG("--suite", ++i, argc);
 			suite_name=argv[i];
 		}
 #endif
-#if HAVE_CU_CURSES
+#ifdef HAVE_CU_CURSES
 		else if (strcmp(argv[i], "--curses") == 0) {
-			i++;
 			curses = 1;
 		}
 #endif
+		else {
+			helper(argv[0]);
+			return -1;
+		}
 	}
 
-	belle_sip_tester_init();
 	ret = belle_sip_tester_run_tests(suite_name, test_name);
 	belle_sip_tester_uninit();
 	return ret;
